from timeutil import hour, minu
import numpy as np
import profile_modifier as modi
import random

'''
Meta data for processing TS
'''
CYCLE_TIME = hour(24)  # 24 hours cycle
PROFILE_INTERVAL_COUNT = CYCLE_TIME / minu(5)  # For each 5 minutes there is one handles point in a workload profile

__WORKLOAD_MAX_RANDOM = 400  # Maximum number of workload profiles

class Mix(object):
    def __init__(self, name, handles, modified):
        # Configuration name
        self.name = name
        
        # Data set 
        self.handles = handles
        
        # Apply modifier to TS handles
        self.modified = modified


class Handle:
    def __init__(self, name, handle_type, modifier=modi.MOD0, scale=(1, 1), shift=0, additive=0):
        self.name = name
        self.htype = handle_type
        
        self.modifier = modifier
        self.scale = scale 
        self.shift = shift
        self.additive = additive

class HandleType:
    def __init__(self, sid, day=None, max_func=np.max):
        self.id = sid
        self.day = day
        self.max = max_func

class Size:
    def __init__(self, name, users):
        self.name = name
        self.users = users

'''
Prefixes used in Times
'''
MUTATION_RAW = 'RAW'
MUTATION_PRAW = 'PRAW'
MUTATION_PUSER = 'PUSER'
MUTATION_PNORM = 'PNORM'
MUTATION_TRACE = 'PTRACE'

'''
Handle types
'''
TYPE_O2_BUSINESS = HandleType(0, max_func=lambda ts:np.percentile(ts, 99))
TYPE_O2_RETAIL = HandleType(1, max_func=lambda ts:np.percentile(ts, 99))
TYPE_SIS = HandleType(2, max_func=lambda ts:np.percentile(ts, 99))
TYPE_SIS_ALL = HandleType(5, max_func=lambda ts:np.percentile(ts, 97))
 
TYPE_SIS_D3 = HandleType(3, day=3)
TYPE_SIS_D8 = HandleType(3, day=8)
TYPE_SIS_D9 = HandleType(3, day=9)
TYPE_SIS_D10 = HandleType(3, day=10)
TYPE_SIS_D26 = HandleType(3, day=26)

'''
Mixes
'''
mix_mkII = [
            Handle('O2_business_ADDLINEORDER', TYPE_O2_BUSINESS),
            Handle('O2_business_ADDORDER', TYPE_O2_BUSINESS),
            Handle('O2_business_CONTRACTEXT', TYPE_O2_BUSINESS),
            Handle('O2_business_SENDMSG', TYPE_O2_BUSINESS),
            Handle('O2_business_UPDATEACCOUNT', TYPE_O2_BUSINESS),
            Handle('O2_business_UPDATEDSS', TYPE_O2_BUSINESS),
            
            Handle('O2_retail_ADDLINEORDER', TYPE_O2_RETAIL),
            Handle('O2_retail_ADDORDER', TYPE_O2_RETAIL),
            Handle('O2_retail_CONTRACTEXT', TYPE_O2_RETAIL),
            Handle('O2_retail_SENDMSG', TYPE_O2_RETAIL),
            Handle('O2_retail_UPDATEACCOUNT', TYPE_O2_RETAIL),
            Handle('O2_retail_UPDATEDSS', TYPE_O2_RETAIL),
                        
            Handle('SIS_5_cpu', TYPE_SIS),
            Handle('SIS_6_cpu', TYPE_SIS),
            Handle('SIS_14_cpu', TYPE_SIS),
            Handle('SIS_19_cpu', TYPE_SIS),
            Handle('SIS_20_cpu', TYPE_SIS),
            Handle('SIS_91_cpu', TYPE_SIS),
            Handle('SIS_87_cpu', TYPE_SIS),
            Handle('SIS_21_cpu', TYPE_SIS),
            Handle('SIS_116_cpu', TYPE_SIS),
            Handle('SIS_125_cpu', TYPE_SIS),
            Handle('SIS_148_cpu', TYPE_SIS),
            Handle('SIS_163_cpu', TYPE_SIS),
            Handle('SIS_165_cpu', TYPE_SIS),
            Handle('SIS_24_cpu', TYPE_SIS),
            Handle('SIS_88_cpu', TYPE_SIS),
            Handle('SIS_93_cpu', TYPE_SIS),
            Handle('SIS_117_cpu', TYPE_SIS),
            Handle('SIS_124_cpu', TYPE_SIS),
            Handle('SIS_147_cpu', TYPE_SIS),
            Handle('SIS_162_cpu', TYPE_SIS),
            Handle('SIS_164_cpu', TYPE_SIS),
            Handle('SIS_166_cpu', TYPE_SIS),
            Handle('SIS_167_cpu', TYPE_SIS),
            Handle('SIS_168_cpu', TYPE_SIS),
            Handle('SIS_179_cpu', TYPE_SIS),
            Handle('SIS_219_cpu', TYPE_SIS),
            Handle('SIS_227_cpu', TYPE_SIS),
            Handle('SIS_228_cpu', TYPE_SIS),
            Handle('SIS_229_cpu', TYPE_SIS),
            Handle('SIS_252_cpu', TYPE_SIS),
            Handle('SIS_254_cpu', TYPE_SIS),
            Handle('SIS_256_cpu', TYPE_SIS),
            Handle('SIS_286_cpu', TYPE_SIS),
            Handle('SIS_284_cpu', TYPE_SIS),
            Handle('SIS_288_cpu', TYPE_SIS),
            Handle('SIS_305_cpu', TYPE_SIS),
            Handle('SIS_316_cpu', TYPE_SIS),
            Handle('SIS_325_cpu', TYPE_SIS),
            Handle('SIS_330_cpu', TYPE_SIS),
            Handle('SIS_335_cpu', TYPE_SIS),
            Handle('SIS_340_cpu', TYPE_SIS),
            Handle('SIS_349_cpu', TYPE_SIS),
            Handle('SIS_357_cpu', TYPE_SIS),
            Handle('SIS_376_cpu', TYPE_SIS),
            Handle('SIS_378_cpu', TYPE_SIS),
            Handle('SIS_413_cpu', TYPE_SIS),
            Handle('SIS_414_cpu', TYPE_SIS),
            Handle('SIS_417_cpu', TYPE_SIS),

            ]

mix_0 = [
            Handle('O2_business_ADDORDER', TYPE_O2_BUSINESS, modi.MOD8, (1.2, 1), minu(10), 20),
            Handle('O2_business_SENDMSG', TYPE_O2_BUSINESS, modi.MOD2, (1.1, 1), minu(-15), 20),
            Handle('O2_business_UPDATEACCOUNT', TYPE_O2_BUSINESS, modi.MOD3, (1.3, 1.1), minu(10), 20),
            Handle('O2_retail_ADDORDER', TYPE_O2_RETAIL, modi.MOD8, (1, 0.8), minu(30), 10),
            Handle('SIS_161_cpu', TYPE_SIS, modi.MOD5, (1.1, 0.9), minu(10), 5),
            Handle('SIS_162_cpu', TYPE_SIS, modi.MOD7, (1, 1), minu(15), 20),
            Handle('SIS_163_cpu', TYPE_SIS, modi.MOD1, (1, 1.3), minu(0), 20),
            Handle('SIS_175_cpu', TYPE_SIS, modi.MOD2, (1.1, 1), minu(-5), 20),
            Handle('SIS_177_cpu', TYPE_SIS, modi.MOD8, (1, 1), minu(20), 20),
            Handle('SIS_179_cpu', TYPE_SIS, modi.MOD4, (1, 1), minu(-7), 20),
            Handle('SIS_188_cpu', TYPE_SIS, modi.MOD5, (1, 1), minu(0), 20),
            Handle('SIS_269_cpu', TYPE_SIS, modi.MOD6, (1, 1.1), minu(1), 20),
            Handle('SIS_298_cpu', TYPE_SIS, modi.MOD7, (1.1, 1), minu(0), 20),
            Handle('SIS_305_cpu', TYPE_SIS, modi.MOD1, (1, 1), minu(-6), 20),
            Handle('SIS_308_cpu', TYPE_SIS, modi.MOD7, (1, 1), minu(5), 20),
            Handle('SIS_310_cpu', TYPE_SIS, modi.MOD3, (1, 0.9), minu(10), 20),
            Handle('SIS_340_cpu', TYPE_SIS, modi.MOD4, (1, 1), minu(15), 20),
            Handle('SIS_393_cpu', TYPE_SIS, modi.MOD3, (1.1, 1), minu(0), 20),
            Handle('SIS_397_cpu', TYPE_SIS, modi.MOD7, (1.05, 1.1), minu(-10), 20),
            Handle('SIS_29_cpu', TYPE_SIS_D3, modi.MOD8, (1, 1), minu(20), 30),
            ]

mix_1 = [
         Handle('SIS_397_cpu', TYPE_SIS, modi.MOD7, (1.1, 1), minu(10), 30),
         Handle('SIS_199_cpu', TYPE_SIS_D8, modi.MOD7, (1.1, 0.9), minu(-5), 50),
         Handle('SIS_207_cpu', TYPE_SIS_D9, modi.MOD4, (1.1, 1), minu(-30), 50),
         Handle('SIS_211_cpu', TYPE_SIS_D9, modi.MOD0, (1.2, 0.8), minu(10), 50),
         Handle('SIS_213_cpu', TYPE_SIS_D9, modi.MOD5, (1, 1.1), minu(20), 30),
         Handle('SIS_216_cpu', TYPE_SIS_D9, modi.MOD5, (1.2, 1), minu(0), 20),
         Handle('SIS_221_cpu', TYPE_SIS_D9, modi.MOD7, (1, 1.3), minu(-10), 30),
         Handle('SIS_222_cpu', TYPE_SIS_D9, modi.MOD1, (1.0, 1), minu(-50), 30),
         Handle('SIS_225_cpu', TYPE_SIS_D9, modi.MOD9, (1.1, 1.3), minu(10), 10),
         Handle('SIS_234_cpu', TYPE_SIS_D9, modi.MOD8, (1, 1), minu(50), 40),
         Handle('SIS_245_cpu', TYPE_SIS_D9, modi.MOD4, (1.1, 1), minu(0), 20),
         Handle('SIS_264_cpu', TYPE_SIS_D9, modi.MOD3, (1, 1), minu(-20), 50),
         Handle('SIS_271_cpu', TYPE_SIS_D9, modi.MOD6, (1.1, 1.4), minu(50), 40),
         Handle('SIS_275_cpu', TYPE_SIS_D9, modi.MOD5, (1.02, 1.3), minu(15), 10),
         Handle('SIS_279_cpu', TYPE_SIS_D9, modi.MOD1, (1.2, 1), minu(30), 10),
         Handle('SIS_344_cpu', TYPE_SIS_D8, modi.MOD2, (1.05, 1), minu(6), 30),
         Handle('SIS_345_cpu', TYPE_SIS_D8, modi.MOD0, (1.3, 1.1), minu(-100), 30),
         Handle('SIS_350_cpu', TYPE_SIS_D8, modi.MOD8, (1, 1), minu(0), 10),
         Handle('SIS_385_cpu', TYPE_SIS_D9, modi.MOD3, (1.03, 1.4), minu(10), 20),
         Handle('SIS_387_cpu', TYPE_SIS_D9, modi.MOD6, (1.1, 1.7), minu(50), 50),
         ]

mix_2 = [
         Handle('O2_business_ADDORDER', TYPE_O2_BUSINESS, modi.MOD1, (1, 1), minu(10), 20),
         Handle('O2_business_SENDMSG', TYPE_O2_BUSINESS, modi.MOD7, (1, 1), minu(-5), 20),
         Handle('O2_business_UPDATEACCOUNT', TYPE_O2_BUSINESS, modi.MOD8, (1.1, 1), minu(20), 20),
         Handle('SIS_163_cpu', TYPE_SIS, modi.MOD4, (1, 1), minu(10), 20),
         Handle('SIS_175_cpu', TYPE_SIS, modi.MOD5, (1, 1.1), minu(-20), 20),
         Handle('SIS_179_cpu', TYPE_SIS, modi.MOD6, (1, 1), minu(-10), 20),
         Handle('SIS_298_cpu', TYPE_SIS, modi.MOD7, (1, 1), minu(0), 20),
         Handle('SIS_310_cpu', TYPE_SIS, modi.MOD8, (1.2, 1), minu(0), 10),
         Handle('SIS_340_cpu', TYPE_SIS, modi.MOD2, (1, 1), minu(5), 10),
         Handle('SIS_29_cpu', TYPE_SIS_D3, modi.MOD3, (1, 1.2), hour(1), 20),
         Handle('SIS_199_cpu', TYPE_SIS_D8, modi.MOD4, (1, 1), minu(10), 20),
         Handle('SIS_211_cpu', TYPE_SIS_D9, modi.MOD4, (1.03, 1.1), minu(0), 20),
         Handle('SIS_216_cpu', TYPE_SIS_D9, modi.MOD7, (1.1, 1), minu(20), 0),
         Handle('SIS_225_cpu', TYPE_SIS_D9, modi.MOD7, (1.3, 1), minu(-15), 20),
         Handle('SIS_234_cpu', TYPE_SIS_D9, modi.MOD1, (1.4, 1.2), minu(0), 20),
         Handle('SIS_264_cpu', TYPE_SIS_D9, modi.MOD2, (1, 1), minu(10), 10),
         Handle('SIS_279_cpu', TYPE_SIS_D9, modi.MOD0, (1.2, 1), minu(20), 40),
         Handle('SIS_345_cpu', TYPE_SIS_D8, modi.MOD8, (1, 1.5), minu(0), 40),
         Handle('SIS_387_cpu', TYPE_SIS_D9, modi.MOD4, (1.3, 1), minu(15), 20),
         Handle('SIS_199_cpu', TYPE_SIS_D8, modi.MOD7, (1.1, 1), minu(0), 20),
         ]

mix_sim = []
for i in xrange(1, 420):
    desc = Handle('SIS_%i_cpu' % i, TYPE_SIS_ALL)
    mix_sim.append(desc)

'''
Configuration for different workload mixes
'''
# MKI MIX0-2
mix0 = Mix('MIX0', mix_0, False)
mix1 = Mix('MIX1', mix_1, False)
mix2 = Mix('MIX2', mix_2, False)

# MKI MIX0-2 modified
mix0m = Mix('MIX0m', mix_0, True)
mix1m = Mix('MIX1m', mix_1, True)
mix2m = Mix('MIX2m', mix_2, True)

# MKII 
mixmkII = Mix('MIXMKII', mix_mkII, False)

# Simulations
mixsim0 = Mix('MIX_SIM0', mix_sim, False)

'''
Scrambler to get random workload profiles
'''
scrabler = []
scrabler.append([i for i in xrange(__WORKLOAD_MAX_RANDOM)])  # identity mapping
scrabler.append([14, 276, 294, 321, 249, 309, 77, 244, 145, 387, 307, 16, 266, 331, 157, 293, 1, 1, 286, 57, 115, 23, 236, 141, 375, 322, 1, 203, 365, 241, 176, 44, 124, 376, 137, 328, 115, 175, 269, 180, 3, 317, 381, 251, 359, 307, 267, 320, 65, 277, 38, 153, 80, 295, 26, 231, 105, 400, 370, 283, 20, 389, 127, 304, 365, 367, 100, 389, 370, 202, 189, 298, 274, 85, 389, 161, 117, 16, 7, 223, 323, 302, 309, 344, 68, 320, 181, 174, 67, 5, 373, 357, 235, 133, 176, 130, 10, 229, 313, 16, 23, 220, 183, 377, 275, 37, 118, 380, 156, 141, 152, 348, 314, 88, 272, 178, 190, 98, 57, 104, 304, 48, 293, 250, 184, 8, 226, 174, 259, 74, 199, 342, 42, 255, 172, 320, 228, 104, 297, 118, 290, 155, 262, 253, 162, 373, 81, 269, 240, 376, 161, 258, 287, 255, 235, 325, 275, 226, 327, 115, 322, 150, 391, 310, 21, 340, 346, 314, 253, 235, 18, 385, 253, 51, 54, 213, 357, 359, 358, 100, 149, 98, 124, 178, 108, 196, 136, 136, 272, 187, 248, 315, 200, 203, 189, 209, 53, 120, 346, 46, 310, 11, 153, 330, 323, 274, 142, 11, 348, 281, 119, 269, 92, 144, 102, 44, 133, 233, 138, 79, 369, 264, 282, 75, 364, 39, 306, 59, 345, 125, 103, 146, 299, 236, 222, 231, 99, 197, 58, 60, 165, 369, 139, 342, 314, 180, 107, 57, 151, 81, 200, 184, 253, 5, 297, 385, 320, 152, 374, 291, 34, 363, 224, 195, 349, 43, 263, 311, 19, 302, 90, 274, 121, 272, 349, 5, 325, 71, 346, 76, 123, 145, 275, 143, 371, 346, 103, 104, 289, 386, 387, 291, 239, 61, 97, 163, 340, 279, 34, 351, 90, 85, 27, 337, 119, 35, 10, 265, 127, 25, 153, 345, 69, 40, 106, 191, 6, 7, 2, 376, 306, 91, 392, 178, 300, 54, 141, 267, 321, 80, 127, 381, 128, 293, 360, 126, 309, 371, 171, 139, 282, 174, 173, 41, 282, 181, 339, 180, 348, 241, 242, 319, 205, 12, 101, 19, 390, 295, 353, 369, 114, 6, 386, 258, 7, 181, 260, 205, 222, 175, 82, 245, 123, 19, 327, 325, 373, 34, 227, 66, 362, 94, 83, 353, 168, 163, 160, 357, 178, 15, 243, 362, 327, 323, 139, 119, 323, 151, 137, 296])
scrabler.append([234, 333, 271, 71, 71, 75, 307, 373, 19, 397, 128, 44, 48, 161, 268, 87, 135, 134, 31, 4, 333, 209, 182, 370, 356, 184, 31, 369, 98, 198, 357, 174, 243, 229, 168, 353, 389, 121, 27, 147, 371, 391, 140, 138, 288, 4, 212, 345, 312, 190, 237, 344, 311, 103, 8, 291, 47, 124, 94, 93, 227, 147, 287, 398, 10, 327, 153, 310, 154, 89, 275, 374, 320, 197, 30, 59, 51, 397, 94, 83, 168, 368, 143, 216, 153, 354, 304, 308, 54, 54, 232, 165, 397, 46, 267, 98, 219, 341, 75, 200, 5, 29, 73, 267, 6, 76, 153, 72, 207, 337, 374, 308, 13, 208, 331, 309, 55, 315, 270, 347, 258, 376, 95, 46, 203, 61, 379, 152, 34, 306, 163, 112, 46, 310, 175, 193, 136, 305, 283, 249, 165, 11, 70, 166, 105, 245, 356, 106, 225, 29, 223, 359, 336, 14, 181, 250, 110, 300, 375, 175, 346, 258, 288, 32, 97, 50, 190, 97, 200, 137, 181, 6, 269, 335, 341, 265, 127, 287, 357, 343, 73, 124, 277, 71, 287, 393, 105, 209, 372, 138, 45, 56, 207, 109, 20, 70, 28, 51, 329, 322, 44, 366, 36, 66, 220, 230, 246, 363, 237, 27, 119, 169, 204, 329, 119, 369, 167, 394, 156, 340, 172, 101, 380, 279, 313, 171, 367, 350, 253, 30, 344, 113, 78, 255, 306, 62, 258, 50, 336, 381, 336, 295, 188, 384, 288, 219, 150, 340, 41, 179, 214, 239, 301, 272, 74, 184, 208, 312, 392, 104, 348, 201, 296, 89, 217, 110, 240, 297, 273, 239, 190, 7, 92, 394, 67, 10, 346, 78, 192, 58, 131, 175, 308, 341, 220, 303, 337, 128, 8, 150, 233, 100, 158, 94, 85, 186, 44, 36, 390, 349, 71, 305, 68, 283, 314, 143, 310, 13, 11, 180, 191, 256, 374, 352, 142, 137, 279, 134, 288, 250, 172, 342, 295, 268, 114, 254, 344, 86, 322, 150, 84, 204, 370, 241, 261, 337, 99, 273, 355, 286, 93, 323, 80, 67, 179, 266, 54, 307, 297, 197, 213, 218, 159, 190, 261, 82, 225, 282, 60, 111, 75, 210, 86, 128, 95, 347, 240, 340, 239, 43, 349, 372, 308, 24, 70, 106, 38, 366, 118, 73, 327, 20, 272, 283, 82, 143, 369, 87, 92, 68, 69, 357, 328, 157, 78, 188, 65, 268, 308, 36])
scrabler.append([96, 214, 352, 318, 392, 55, 62, 189, 382, 248, 129, 158, 42, 165, 385, 116, 178, 307, 75, 255, 159, 230, 98, 64, 136, 338, 132, 124, 135, 82, 40, 37, 149, 180, 50, 214, 93, 299, 318, 83, 268, 95, 206, 68, 364, 134, 152, 309, 335, 325, 78, 241, 205, 105, 231, 301, 396, 219, 17, 90, 366, 172, 223, 229, 240, 121, 341, 370, 245, 125, 198, 137, 356, 172, 368, 376, 375, 28, 342, 302, 164, 127, 337, 288, 336, 394, 311, 154, 94, 23, 250, 280, 68, 199, 390, 10, 188, 339, 264, 64, 391, 67, 159, 182, 78, 118, 39, 112, 299, 332, 150, 214, 180, 7, 347, 360, 337, 398, 274, 343, 318, 8, 307, 56, 309, 311, 219, 208, 284, 277, 209, 136, 153, 289, 15, 104, 90, 252, 249, 261, 228, 272, 207, 183, 54, 46, 157, 39, 35, 100, 271, 398, 286, 43, 326, 339, 108, 398, 138, 231, 313, 182, 378, 150, 330, 107, 238, 45, 117, 193, 97, 69, 94, 129, 232, 92, 308, 400, 396, 67, 176, 343, 80, 400, 318, 328, 381, 198, 276, 166, 75, 400, 312, 4, 383, 379, 263, 229, 314, 212, 244, 152, 279, 360, 275, 116, 77, 124, 338, 90, 116, 264, 307, 45, 244, 234, 184, 281, 102, 57, 121, 199, 243, 83, 75, 398, 351, 99, 111, 183, 270, 241, 200, 269, 391, 62, 250, 149, 393, 130, 165, 295, 308, 366, 108, 172, 292, 41, 266, 262, 20, 82, 190, 183, 126, 247, 241, 362, 360, 233, 198, 306, 392, 38, 117, 127, 138, 254, 243, 154, 396, 64, 36, 54, 362, 258, 395, 297, 31, 192, 211, 149, 38, 103, 101, 11, 194, 326, 34, 234, 3, 207, 150, 298, 257, 326, 276, 217, 317, 104, 60, 322, 0, 364, 109, 332, 266, 249, 92, 315, 190, 145, 139, 302, 246, 315, 121, 289, 380, 36, 118, 317, 326, 341, 311, 195, 94, 262, 134, 135, 313, 71, 189, 247, 49, 131, 319, 260, 172, 22, 306, 190, 220, 12, 48, 364, 177, 311, 115, 273, 394, 183, 381, 131, 171, 25, 288, 176, 150, 161, 182, 373, 201, 115, 294, 218, 165, 253, 171, 107, 122, 374, 333, 25, 272, 339, 174, 212, 114, 185, 57, 116, 353, 163, 118, 335, 317, 62, 45, 105, 293, 259, 348, 286, 34, 24, 376, 369, 28, 244])
scrabler.append([284, 15, 315, 376, 107, 381, 8, 42, 189, 232, 124, 184, 331, 125, 186, 194, 189, 239, 354, 59, 255, 393, 346, 214, 318, 385, 371, 330, 113, 184, 259, 312, 190, 114, 16, 298, 25, 20, 5, 258, 69, 321, 112, 40, 123, 269, 303, 122, 367, 318, 222, 214, 96, 249, 49, 318, 254, 142, 144, 380, 82, 83, 100, 207, 331, 178, 385, 189, 242, 304, 55, 300, 304, 221, 263, 102, 215, 20, 297, 230, 333, 201, 65, 100, 259, 140, 62, 76, 83, 246, 174, 43, 179, 9, 385, 329, 35, 372, 340, 91, 230, 241, 39, 144, 369, 75, 215, 35, 66, 325, 101, 349, 44, 392, 315, 57, 115, 327, 12, 65, 107, 318, 273, 269, 238, 273, 222, 196, 335, 333, 157, 228, 323, 377, 274, 19, 380, 129, 159, 46, 71, 154, 387, 256, 377, 243, 361, 74, 367, 134, 102, 52, 78, 224, 190, 341, 216, 53, 9, 203, 299, 38, 375, 149, 15, 396, 266, 160, 196, 189, 78, 100, 256, 98, 217, 335, 373, 138, 124, 235, 265, 301, 44, 112, 14, 307, 354, 200, 203, 260, 151, 228, 256, 377, 308, 275, 331, 239, 331, 360, 26, 79, 200, 1, 49, 326, 103, 147, 36, 199, 188, 147, 71, 318, 157, 32, 140, 311, 275, 248, 106, 247, 160, 182, 347, 361, 342, 84, 303, 108, 358, 288, 235, 196, 83, 112, 347, 370, 355, 388, 372, 157, 129, 24, 151, 390, 233, 49, 359, 208, 357, 369, 164, 317, 220, 102, 165, 200, 61, 294, 191, 204, 116, 352, 147, 229, 72, 117, 335, 109, 125, 258, 219, 213, 195, 337, 127, 41, 96, 248, 111, 121, 214, 387, 100, 318, 195, 176, 391, 216, 202, 299, 20, 6, 180, 289, 7, 256, 8, 122, 54, 158, 342, 63, 191, 144, 2, 48, 175, 212, 307, 239, 18, 14, 303, 297, 265, 194, 55, 11, 191, 371, 155, 207, 83, 161, 89, 233, 139, 113, 12, 397, 363, 77, 96, 165, 271, 294, 123, 329, 73, 386, 192, 1, 270, 143, 218, 9, 74, 373, 254, 94, 143, 26, 350, 152, 313, 331, 367, 362, 141, 124, 165, 321, 228, 165, 179, 375, 101, 143, 291, 361, 82, 131, 369, 47, 105, 394, 320, 164, 283, 393, 329, 255, 169, 315, 249, 62, 263, 53, 66, 391, 314, 75, 156, 341, 103, 14, 340, 292])
scrabler.append([11, 400, 365, 353, 235, 399, 62, 373, 75, 43, 234, 230, 296, 97, 259, 29, 207, 250, 180, 131, 4, 113, 180, 337, 104, 397, 69, 131, 281, 182, 295, 392, 374, 275, 227, 151, 245, 378, 370, 142, 241, 111, 3, 388, 207, 294, 70, 375, 57, 165, 142, 91, 119, 178, 176, 298, 249, 374, 18, 137, 188, 243, 50, 120, 226, 328, 89, 143, 209, 368, 113, 187, 245, 96, 186, 316, 147, 33, 232, 56, 77, 345, 39, 142, 80, 356, 70, 296, 227, 360, 400, 261, 24, 180, 0, 237, 339, 326, 72, 254, 95, 226, 39, 129, 197, 221, 62, 216, 15, 347, 22, 12, 98, 391, 254, 327, 378, 154, 53, 276, 379, 390, 88, 387, 204, 217, 221, 18, 119, 60, 192, 373, 75, 181, 278, 141, 388, 0, 154, 260, 308, 267, 372, 25, 57, 115, 342, 261, 207, 82, 95, 6, 288, 271, 188, 239, 287, 218, 75, 355, 87, 63, 136, 25, 136, 74, 309, 162, 383, 353, 299, 264, 308, 237, 187, 239, 201, 215, 257, 370, 184, 242, 254, 379, 204, 252, 271, 85, 255, 16, 190, 272, 322, 160, 282, 375, 31, 344, 265, 235, 40, 116, 345, 62, 283, 17, 152, 9, 1, 223, 260, 51, 232, 112, 277, 101, 234, 325, 43, 162, 13, 184, 154, 383, 88, 294, 45, 359, 137, 256, 295, 297, 226, 303, 56, 195, 198, 383, 87, 169, 218, 131, 252, 291, 140, 56, 294, 342, 209, 341, 117, 49, 365, 384, 174, 63, 17, 205, 100, 119, 144, 247, 131, 344, 339, 276, 237, 232, 346, 236, 264, 281, 325, 8, 149, 32, 48, 209, 96, 150, 71, 303, 175, 181, 206, 189, 193, 24, 41, 397, 334, 20, 143, 358, 399, 300, 162, 388, 202, 186, 101, 81, 129, 234, 104, 350, 11, 226, 12, 209, 174, 225, 107, 254, 344, 392, 201, 24, 207, 252, 327, 36, 109, 100, 267, 215, 367, 240, 51, 68, 300, 199, 257, 318, 347, 81, 280, 53, 238, 280, 21, 255, 241, 197, 233, 260, 103, 130, 259, 208, 332, 378, 39, 183, 120, 297, 302, 118, 180, 181, 200, 280, 217, 89, 207, 324, 32, 349, 242, 181, 53, 196, 78, 32, 87, 170, 139, 77, 357, 136, 235, 245, 247, 272, 149, 93, 135, 4, 79, 233, 22, 45, 265, 239, 289, 157, 143, 86, 228, 174])
scrabler.append([193, 82, 348, 124, 224, 268, 275, 45, 49, 118, 184, 357, 369, 346, 11, 243, 348, 29, 279, 95, 183, 279, 127, 198, 22, 260, 16, 90, 78, 257, 342, 99, 380, 216, 97, 347, 27, 210, 387, 6, 232, 197, 252, 288, 317, 123, 325, 131, 359, 278, 367, 117, 300, 70, 37, 81, 62, 210, 314, 310, 175, 301, 306, 182, 266, 293, 353, 75, 116, 317, 32, 52, 62, 95, 275, 228, 367, 248, 101, 386, 338, 302, 184, 394, 76, 296, 247, 11, 390, 296, 396, 47, 208, 199, 232, 176, 163, 173, 223, 157, 117, 241, 385, 219, 123, 272, 228, 347, 82, 372, 333, 122, 346, 42, 283, 15, 269, 251, 9, 326, 65, 68, 53, 30, 130, 159, 356, 190, 278, 309, 357, 189, 140, 66, 94, 345, 202, 32, 342, 43, 278, 328, 242, 275, 350, 154, 88, 113, 386, 336, 283, 364, 86, 205, 111, 320, 400, 346, 165, 3, 139, 215, 379, 311, 147, 251, 344, 353, 181, 74, 256, 229, 119, 224, 324, 188, 222, 10, 120, 81, 162, 212, 119, 80, 109, 250, 121, 120, 164, 183, 60, 267, 237, 279, 6, 205, 137, 201, 149, 3, 380, 399, 304, 55, 252, 114, 211, 91, 132, 17, 6, 196, 327, 181, 363, 40, 78, 227, 345, 211, 351, 309, 151, 180, 43, 398, 247, 207, 119, 149, 133, 260, 220, 306, 249, 259, 350, 130, 22, 89, 6, 1, 312, 347, 199, 220, 99, 210, 121, 301, 348, 371, 50, 146, 325, 30, 102, 142, 81, 199, 137, 362, 304, 56, 296, 61, 228, 362, 315, 9, 15, 50, 21, 310, 333, 150, 187, 119, 353, 77, 133, 69, 211, 157, 332, 291, 280, 356, 345, 374, 171, 255, 203, 154, 66, 225, 77, 328, 38, 371, 75, 3, 54, 292, 15, 133, 246, 91, 197, 395, 14, 332, 298, 223, 225, 104, 81, 6, 133, 61, 345, 323, 281, 263, 383, 19, 159, 102, 199, 225, 50, 314, 57, 166, 186, 179, 229, 376, 359, 33, 219, 255, 174, 160, 140, 220, 53, 6, 394, 107, 130, 148, 121, 317, 293, 283, 125, 130, 320, 3, 216, 158, 97, 81, 32, 240, 172, 223, 39, 273, 376, 40, 332, 121, 342, 148, 261, 273, 82, 242, 375, 320, 267, 287, 230, 290, 223, 107, 315, 359, 147, 152, 373, 192, 302, 230, 71, 147, 258, 122])
scrabler.append([301, 195, 283, 160, 297, 217, 102, 377, 373, 21, 64, 231, 374, 172, 349, 93, 221, 304, 66, 228, 115, 129, 10, 290, 70, 387, 185, 75, 161, 365, 143, 50, 275, 8, 207, 378, 278, 24, 45, 339, 236, 339, 244, 332, 257, 108, 132, 268, 272, 166, 69, 123, 94, 117, 386, 110, 207, 191, 82, 150, 50, 331, 54, 293, 53, 182, 94, 231, 230, 280, 148, 386, 313, 53, 76, 184, 342, 172, 122, 395, 357, 35, 344, 149, 325, 24, 9, 195, 26, 17, 221, 352, 382, 390, 368, 400, 348, 7, 187, 282, 14, 145, 237, 176, 331, 86, 70, 348, 106, 137, 218, 209, 164, 253, 10, 97, 383, 290, 139, 307, 195, 50, 360, 23, 3, 115, 365, 383, 316, 135, 352, 245, 21, 253, 220, 232, 196, 238, 63, 354, 170, 86, 357, 61, 173, 400, 136, 349, 249, 323, 51, 346, 39, 238, 37, 209, 351, 302, 242, 42, 188, 115, 70, 35, 123, 321, 57, 261, 379, 219, 170, 63, 66, 18, 172, 357, 154, 161, 9, 116, 55, 27, 360, 165, 55, 262, 390, 54, 60, 373, 77, 399, 162, 301, 318, 121, 397, 204, 117, 171, 250, 147, 240, 123, 173, 372, 183, 177, 199, 121, 318, 365, 109, 254, 330, 347, 137, 307, 308, 323, 369, 8, 83, 30, 150, 315, 300, 373, 398, 362, 319, 217, 118, 162, 88, 159, 395, 68, 367, 134, 68, 186, 216, 258, 371, 280, 153, 192, 151, 339, 114, 371, 120, 397, 32, 278, 310, 2, 45, 283, 14, 358, 310, 285, 124, 338, 362, 219, 172, 62, 299, 177, 151, 109, 260, 349, 0, 118, 199, 364, 324, 271, 7, 218, 247, 250, 336, 42, 147, 380, 6, 37, 226, 373, 254, 313, 388, 78, 18, 360, 18, 89, 114, 368, 135, 43, 295, 294, 376, 32, 54, 8, 121, 324, 351, 356, 311, 314, 135, 193, 240, 374, 156, 184, 319, 155, 15, 322, 330, 33, 256, 90, 325, 319, 294, 224, 95, 101, 11, 128, 66, 348, 304, 235, 17, 300, 53, 390, 228, 102, 56, 281, 81, 165, 299, 170, 56, 285, 182, 389, 391, 88, 197, 75, 173, 312, 41, 227, 91, 344, 262, 183, 339, 215, 27, 348, 336, 393, 92, 151, 271, 387, 165, 315, 23, 347, 116, 205, 336, 79, 9, 165, 287, 280, 283, 349, 398, 307, 305, 29])
scrabler.append([77, 251, 113, 35, 94, 269, 315, 89, 149, 5, 363, 31, 191, 64, 188, 14, 314, 107, 331, 344, 53, 71, 265, 88, 284, 135, 24, 255, 145, 182, 326, 50, 334, 180, 212, 253, 181, 126, 195, 23, 135, 255, 175, 270, 255, 284, 315, 382, 25, 259, 172, 49, 32, 271, 221, 14, 141, 239, 141, 348, 261, 103, 141, 147, 249, 366, 367, 33, 259, 78, 384, 181, 152, 106, 186, 337, 218, 272, 357, 78, 309, 162, 249, 378, 212, 325, 221, 67, 266, 153, 191, 366, 264, 316, 102, 329, 289, 92, 385, 177, 109, 337, 2, 87, 52, 257, 314, 98, 64, 228, 364, 75, 5, 373, 29, 373, 320, 399, 107, 177, 3, 234, 163, 39, 78, 76, 340, 265, 393, 236, 30, 61, 306, 340, 131, 158, 342, 300, 161, 278, 236, 301, 201, 275, 283, 216, 261, 4, 85, 386, 293, 103, 323, 356, 185, 79, 313, 194, 376, 235, 277, 103, 246, 126, 389, 271, 206, 274, 192, 392, 362, 365, 265, 385, 344, 218, 279, 337, 314, 178, 68, 64, 96, 198, 32, 13, 267, 315, 339, 197, 323, 236, 227, 334, 232, 303, 303, 362, 41, 315, 202, 187, 339, 329, 175, 49, 157, 129, 369, 48, 24, 83, 272, 161, 153, 328, 213, 42, 329, 34, 290, 290, 388, 84, 89, 67, 394, 163, 237, 391, 162, 364, 112, 263, 151, 228, 199, 346, 226, 280, 288, 142, 204, 239, 111, 314, 106, 288, 66, 26, 382, 138, 42, 248, 256, 166, 161, 262, 229, 396, 11, 211, 55, 147, 278, 167, 183, 364, 376, 376, 5, 281, 203, 307, 176, 220, 319, 262, 141, 288, 205, 188, 204, 8, 112, 218, 105, 234, 252, 85, 19, 139, 307, 191, 13, 1, 323, 5, 169, 15, 216, 343, 92, 185, 219, 202, 50, 320, 12, 94, 102, 243, 36, 21, 122, 385, 33, 336, 122, 131, 62, 199, 302, 264, 275, 291, 254, 117, 328, 130, 370, 95, 378, 241, 87, 319, 245, 191, 136, 359, 377, 124, 190, 158, 110, 341, 93, 236, 312, 247, 332, 244, 80, 208, 134, 258, 272, 25, 275, 30, 91, 394, 244, 397, 192, 105, 350, 43, 78, 290, 248, 392, 135, 343, 235, 145, 66, 139, 321, 194, 173, 288, 18, 56, 8, 376, 295, 275, 362, 254, 30, 184, 60, 82, 227, 36, 0, 245, 190, 342])
scrabler.append([17, 202, 63, 389, 353, 294, 171, 243, 325, 100, 36, 210, 260, 179, 106, 162, 38, 177, 240, 36, 123, 136, 174, 41, 114, 347, 32, 378, 163, 323, 86, 212, 38, 109, 171, 243, 88, 29, 395, 376, 20, 119, 20, 49, 96, 126, 41, 86, 331, 273, 79, 226, 319, 23, 256, 245, 318, 58, 266, 213, 265, 145, 384, 272, 283, 314, 171, 28, 266, 16, 303, 55, 179, 6, 396, 78, 325, 12, 16, 106, 124, 318, 317, 348, 288, 192, 369, 336, 212, 159, 297, 367, 86, 307, 346, 299, 58, 143, 277, 301, 185, 86, 263, 17, 120, 318, 177, 180, 316, 72, 297, 127, 351, 385, 121, 109, 216, 8, 316, 301, 82, 137, 21, 250, 382, 238, 190, 328, 308, 396, 225, 362, 41, 221, 70, 280, 292, 41, 190, 330, 58, 47, 159, 69, 287, 377, 154, 355, 352, 164, 336, 138, 95, 325, 137, 48, 391, 34, 347, 24, 83, 267, 70, 389, 92, 30, 63, 2, 181, 89, 65, 66, 163, 392, 92, 147, 153, 85, 257, 167, 137, 394, 282, 65, 341, 183, 89, 369, 210, 360, 155, 48, 148, 63, 56, 191, 96, 106, 310, 135, 50, 209, 66, 350, 38, 85, 286, 52, 256, 33, 370, 232, 243, 385, 2, 2, 29, 190, 318, 267, 296, 149, 318, 250, 252, 199, 75, 224, 252, 3, 61, 303, 265, 207, 304, 97, 191, 58, 380, 88, 133, 268, 130, 310, 379, 299, 263, 301, 55, 356, 129, 125, 95, 218, 278, 308, 98, 9, 243, 37, 275, 84, 400, 100, 89, 141, 279, 208, 87, 154, 276, 219, 20, 109, 141, 15, 15, 326, 7, 235, 361, 4, 69, 349, 233, 390, 37, 380, 120, 36, 344, 23, 46, 292, 254, 26, 349, 316, 129, 298, 23, 24, 111, 365, 30, 153, 193, 355, 395, 93, 369, 368, 167, 296, 333, 223, 100, 180, 94, 113, 316, 68, 103, 151, 347, 115, 76, 55, 135, 156, 34, 227, 375, 126, 315, 350, 259, 269, 98, 180, 64, 42, 65, 366, 224, 325, 28, 326, 363, 334, 152, 383, 87, 107, 309, 61, 61, 61, 68, 2, 247, 377, 354, 134, 292, 49, 344, 328, 263, 59, 306, 118, 289, 22, 94, 20, 319, 21, 18, 332, 162, 14, 56, 323, 358, 255, 244, 224, 57, 82, 355, 274, 176, 29, 148, 60, 252, 16, 176, 87])
scrabler.append([74, 161, 331, 376, 46, 34, 18, 313, 303, 247, 316, 39, 377, 201, 184, 333, 377, 190, 308, 245, 317, 35, 339, 24, 299, 133, 289, 350, 305, 180, 308, 153, 84, 177, 159, 342, 80, 76, 84, 298, 37, 227, 128, 109, 249, 72, 229, 39, 183, 296, 332, 107, 200, 57, 123, 37, 359, 18, 370, 93, 51, 398, 98, 7, 369, 112, 305, 320, 228, 297, 186, 37, 181, 294, 62, 394, 49, 144, 224, 8, 175, 43, 253, 170, 38, 225, 211, 285, 60, 72, 171, 135, 160, 273, 9, 250, 31, 260, 106, 86, 0, 216, 344, 346, 248, 101, 179, 29, 296, 210, 91, 149, 205, 301, 299, 219, 151, 361, 248, 144, 167, 65, 395, 297, 289, 390, 191, 258, 150, 191, 17, 372, 267, 309, 149, 130, 364, 397, 175, 221, 352, 223, 237, 80, 195, 100, 174, 59, 396, 121, 97, 384, 22, 315, 173, 138, 330, 109, 237, 311, 221, 106, 95, 335, 400, 122, 307, 125, 385, 58, 86, 266, 360, 99, 363, 260, 390, 43, 156, 146, 28, 66, 380, 274, 303, 387, 16, 7, 317, 364, 298, 248, 53, 325, 314, 3, 210, 327, 379, 159, 138, 135, 353, 378, 289, 276, 7, 94, 17, 313, 382, 131, 141, 316, 269, 97, 247, 246, 305, 229, 117, 348, 333, 182, 177, 349, 367, 205, 147, 221, 261, 384, 126, 381, 255, 326, 171, 318, 181, 250, 278, 369, 377, 176, 73, 139, 211, 80, 214, 244, 268, 10, 235, 80, 386, 281, 118, 90, 121, 393, 37, 15, 304, 292, 294, 214, 350, 254, 357, 376, 11, 143, 175, 8, 8, 168, 350, 325, 190, 398, 359, 99, 178, 269, 374, 167, 287, 388, 123, 9, 101, 188, 19, 357, 158, 257, 275, 216, 109, 265, 394, 1, 360, 22, 133, 201, 101, 125, 7, 136, 36, 263, 115, 160, 111, 161, 153, 389, 8, 298, 400, 50, 71, 123, 125, 257, 295, 167, 310, 396, 144, 322, 362, 237, 104, 229, 247, 2, 4, 273, 219, 44, 340, 192, 357, 298, 139, 311, 147, 110, 257, 155, 237, 194, 152, 366, 288, 386, 132, 106, 156, 246, 39, 394, 6, 10, 263, 397, 97, 153, 65, 159, 89, 77, 214, 363, 5, 236, 309, 259, 315, 333, 266, 66, 324, 319, 46, 128, 19, 43, 268, 0, 393, 53, 399, 204, 92, 136, 340, 307])
scrabler.append([214, 345, 33, 137, 390, 128, 381, 8, 44, 221, 99, 246, 191, 291, 158, 105, 288, 30, 278, 15, 271, 191, 300, 337, 124, 218, 115, 157, 259, 8, 242, 12, 112, 217, 185, 113, 189, 146, 348, 202, 283, 78, 400, 5, 230, 215, 201, 201, 94, 187, 138, 363, 289, 23, 358, 301, 62, 122, 367, 183, 58, 128, 3, 168, 38, 394, 207, 6, 93, 357, 277, 196, 198, 300, 216, 202, 377, 166, 158, 388, 181, 100, 332, 85, 297, 331, 63, 70, 354, 46, 32, 109, 96, 265, 103, 184, 313, 260, 149, 150, 244, 52, 63, 395, 299, 302, 146, 363, 400, 251, 395, 6, 307, 146, 270, 371, 36, 307, 95, 375, 264, 332, 339, 357, 172, 170, 280, 360, 2, 302, 68, 70, 191, 193, 29, 386, 13, 211, 348, 100, 359, 252, 150, 73, 164, 248, 91, 369, 332, 199, 6, 17, 11, 84, 355, 48, 87, 213, 291, 74, 76, 224, 346, 183, 371, 119, 109, 266, 22, 376, 151, 378, 253, 92, 227, 135, 39, 161, 79, 78, 76, 105, 273, 88, 229, 347, 229, 29, 266, 216, 301, 203, 234, 258, 273, 183, 7, 143, 207, 319, 215, 155, 196, 41, 214, 110, 308, 331, 164, 35, 264, 211, 111, 249, 252, 201, 63, 205, 231, 213, 77, 312, 291, 41, 135, 159, 360, 234, 185, 305, 377, 15, 132, 51, 289, 122, 274, 168, 84, 1, 115, 368, 113, 223, 114, 312, 127, 37, 271, 44, 299, 66, 109, 234, 258, 107, 330, 132, 309, 234, 390, 167, 98, 340, 323, 119, 245, 256, 318, 371, 317, 317, 163, 26, 235, 308, 90, 237, 58, 344, 391, 376, 305, 6, 274, 162, 89, 360, 140, 259, 169, 348, 98, 112, 150, 134, 20, 226, 32, 20, 36, 196, 93, 167, 168, 377, 171, 342, 63, 85, 148, 314, 277, 380, 364, 144, 255, 34, 200, 296, 185, 226, 373, 313, 377, 211, 25, 207, 375, 1, 278, 275, 50, 248, 293, 288, 44, 116, 254, 393, 332, 26, 237, 286, 323, 165, 22, 244, 165, 324, 361, 151, 172, 146, 353, 370, 125, 92, 311, 395, 224, 39, 165, 217, 251, 94, 133, 205, 176, 14, 237, 135, 162, 82, 350, 69, 72, 181, 133, 171, 178, 335, 14, 189, 48, 178, 348, 150, 252, 234, 388, 154, 332, 9, 259, 325, 324, 196, 257, 359])
scrabler.append([88, 22, 318, 80, 20, 395, 354, 307, 27, 153, 353, 399, 51, 202, 105, 29, 382, 3, 235, 17, 145, 89, 271, 372, 190, 107, 18, 50, 197, 136, 233, 205, 169, 134, 103, 250, 299, 371, 89, 29, 303, 41, 55, 356, 198, 274, 308, 312, 275, 330, 157, 300, 292, 160, 247, 244, 390, 344, 244, 313, 39, 351, 378, 128, 185, 173, 12, 92, 185, 280, 352, 224, 78, 206, 169, 74, 318, 281, 20, 317, 0, 266, 254, 152, 57, 285, 375, 90, 190, 172, 3, 318, 328, 113, 183, 119, 303, 110, 210, 328, 46, 57, 325, 267, 374, 264, 82, 337, 94, 370, 324, 292, 128, 363, 19, 59, 301, 390, 376, 376, 228, 142, 137, 65, 385, 325, 147, 366, 57, 118, 379, 11, 79, 88, 138, 116, 269, 47, 83, 301, 23, 42, 278, 230, 244, 376, 66, 102, 71, 240, 370, 368, 179, 394, 289, 291, 14, 268, 95, 42, 138, 89, 137, 237, 80, 182, 316, 40, 199, 108, 235, 37, 360, 349, 75, 71, 5, 230, 296, 26, 112, 241, 175, 295, 79, 234, 79, 397, 330, 338, 79, 15, 145, 146, 357, 187, 100, 180, 113, 303, 156, 321, 375, 4, 268, 28, 222, 66, 101, 208, 309, 210, 380, 250, 368, 33, 188, 10, 195, 164, 61, 20, 337, 177, 394, 25, 193, 98, 105, 134, 118, 234, 80, 282, 286, 231, 250, 285, 374, 82, 253, 372, 250, 293, 286, 323, 235, 104, 333, 303, 95, 222, 323, 157, 303, 216, 395, 322, 289, 206, 384, 33, 108, 280, 214, 164, 212, 248, 82, 36, 213, 382, 133, 383, 210, 111, 170, 152, 246, 66, 88, 187, 94, 251, 84, 135, 399, 14, 177, 320, 234, 193, 162, 327, 141, 188, 359, 225, 76, 256, 181, 104, 89, 19, 186, 236, 7, 302, 88, 310, 234, 266, 354, 245, 214, 317, 355, 221, 181, 159, 356, 354, 99, 8, 240, 269, 140, 267, 160, 66, 8, 378, 254, 7, 314, 245, 358, 134, 97, 59, 214, 153, 60, 392, 215, 342, 178, 321, 262, 268, 301, 122, 165, 296, 127, 258, 255, 55, 159, 348, 260, 381, 22, 144, 52, 327, 258, 198, 48, 242, 112, 25, 322, 294, 100, 99, 172, 313, 116, 376, 280, 114, 302, 180, 168, 239, 171, 188, 321, 196, 16, 197, 207, 17, 61, 28, 87, 55, 48, 73])
scrabler.append([375, 68, 32, 110, 87, 387, 264, 25, 220, 288, 386, 40, 48, 273, 392, 366, 283, 157, 278, 312, 82, 55, 301, 357, 158, 202, 284, 73, 8, 392, 365, 73, 358, 243, 223, 146, 304, 85, 359, 392, 344, 241, 382, 322, 30, 129, 218, 338, 195, 387, 97, 353, 47, 96, 179, 283, 278, 244, 184, 394, 118, 242, 304, 304, 26, 53, 387, 103, 189, 147, 199, 213, 39, 127, 10, 323, 151, 217, 308, 264, 183, 155, 282, 182, 186, 38, 386, 299, 316, 147, 122, 10, 80, 27, 317, 71, 334, 206, 57, 56, 158, 359, 103, 153, 128, 314, 234, 289, 130, 29, 378, 3, 213, 316, 350, 57, 383, 73, 376, 136, 93, 290, 155, 167, 393, 291, 329, 163, 78, 344, 68, 63, 0, 34, 373, 146, 372, 326, 108, 296, 126, 300, 247, 245, 394, 284, 226, 390, 245, 206, 316, 41, 333, 151, 359, 92, 43, 391, 68, 164, 385, 395, 394, 216, 373, 70, 49, 111, 75, 50, 36, 371, 81, 185, 53, 49, 383, 376, 61, 374, 371, 247, 232, 109, 28, 29, 82, 260, 66, 145, 293, 67, 351, 190, 14, 179, 237, 102, 297, 196, 291, 11, 193, 9, 99, 176, 2, 134, 162, 310, 214, 386, 23, 276, 101, 7, 74, 130, 391, 151, 8, 184, 342, 319, 25, 142, 227, 5, 149, 379, 115, 359, 138, 164, 377, 144, 312, 329, 111, 122, 163, 191, 360, 7, 12, 157, 67, 370, 373, 228, 295, 237, 226, 142, 130, 36, 399, 129, 15, 26, 346, 359, 157, 198, 54, 338, 107, 187, 23, 81, 56, 328, 390, 57, 115, 211, 394, 391, 92, 310, 67, 319, 140, 216, 299, 347, 301, 395, 170, 57, 100, 255, 234, 51, 56, 59, 207, 325, 268, 106, 212, 163, 348, 88, 129, 270, 0, 126, 50, 331, 134, 77, 68, 53, 320, 321, 2, 231, 86, 289, 70, 107, 362, 151, 196, 14, 75, 390, 41, 32, 146, 367, 368, 277, 304, 293, 27, 148, 203, 285, 305, 155, 323, 244, 103, 346, 46, 381, 54, 15, 2, 111, 118, 141, 39, 327, 342, 116, 351, 222, 6, 158, 304, 145, 383, 75, 204, 146, 34, 294, 312, 243, 265, 149, 36, 278, 32, 283, 264, 201, 328, 255, 38, 246, 179, 221, 200, 400, 36, 160, 31, 2, 148, 360, 77, 318, 79, 327, 139, 55])
scrabler.append([88, 312, 240, 18, 67, 3, 27, 266, 339, 242, 24, 320, 308, 245, 271, 377, 126, 384, 132, 94, 295, 134, 59, 318, 109, 1, 387, 58, 366, 76, 377, 276, 158, 212, 375, 332, 4, 76, 5, 221, 33, 243, 331, 286, 270, 181, 157, 191, 322, 20, 360, 9, 122, 320, 379, 167, 5, 202, 252, 43, 354, 151, 342, 337, 332, 67, 67, 91, 82, 367, 293, 131, 340, 89, 319, 253, 182, 276, 212, 16, 291, 333, 201, 137, 276, 177, 86, 154, 77, 298, 312, 172, 390, 190, 257, 302, 161, 358, 348, 345, 246, 189, 319, 54, 355, 254, 366, 309, 141, 178, 253, 29, 25, 202, 341, 47, 60, 26, 273, 70, 30, 339, 353, 277, 302, 289, 165, 393, 46, 13, 268, 58, 300, 132, 391, 374, 89, 39, 25, 129, 93, 6, 298, 325, 376, 145, 139, 398, 388, 196, 85, 176, 64, 361, 204, 210, 391, 94, 254, 364, 144, 325, 6, 194, 49, 228, 238, 125, 187, 45, 388, 77, 277, 392, 158, 199, 144, 147, 92, 313, 263, 287, 110, 81, 69, 289, 54, 145, 389, 216, 18, 337, 110, 339, 60, 50, 44, 191, 351, 13, 294, 293, 24, 388, 130, 330, 326, 103, 154, 16, 113, 313, 72, 4, 88, 391, 288, 276, 189, 60, 386, 96, 242, 307, 92, 210, 80, 177, 379, 11, 67, 357, 281, 14, 102, 307, 392, 349, 269, 82, 331, 55, 69, 313, 400, 200, 30, 118, 205, 275, 98, 88, 355, 235, 70, 40, 117, 221, 290, 170, 175, 249, 81, 318, 83, 190, 64, 99, 309, 288, 217, 98, 372, 23, 394, 154, 358, 76, 18, 68, 191, 310, 111, 361, 264, 68, 358, 31, 399, 345, 339, 202, 351, 238, 217, 68, 109, 59, 307, 87, 361, 394, 219, 260, 360, 321, 119, 253, 334, 360, 92, 105, 18, 367, 323, 235, 225, 375, 142, 157, 11, 16, 252, 165, 251, 222, 283, 2, 116, 345, 215, 134, 57, 298, 272, 92, 48, 323, 50, 66, 68, 330, 366, 75, 168, 91, 131, 239, 306, 240, 310, 270, 299, 45, 289, 52, 159, 220, 289, 385, 152, 129, 211, 363, 305, 305, 248, 130, 351, 106, 142, 191, 314, 249, 282, 33, 177, 226, 240, 300, 215, 359, 269, 44, 144, 325, 22, 44, 268, 74, 342, 153, 192, 219, 297, 112, 157, 391, 290, 196])
scrabler.append([44, 299, 134, 149, 205, 188, 388, 385, 64, 264, 119, 243, 199, 248, 188, 228, 3, 217, 379, 283, 216, 141, 64, 65, 34, 275, 308, 114, 142, 371, 312, 330, 36, 184, 24, 157, 155, 323, 67, 202, 56, 81, 135, 29, 153, 251, 8, 165, 243, 141, 289, 273, 257, 399, 303, 350, 14, 315, 99, 131, 333, 70, 287, 352, 296, 208, 314, 119, 52, 300, 170, 318, 202, 236, 110, 18, 14, 242, 198, 28, 347, 139, 84, 385, 171, 370, 73, 205, 110, 263, 31, 119, 124, 307, 25, 74, 179, 47, 42, 285, 247, 187, 241, 158, 47, 169, 151, 98, 398, 274, 291, 317, 293, 352, 231, 37, 132, 298, 16, 26, 161, 14, 278, 35, 224, 97, 36, 302, 223, 282, 308, 64, 386, 110, 37, 366, 107, 134, 358, 193, 59, 381, 353, 132, 363, 249, 15, 134, 255, 244, 150, 348, 241, 395, 238, 350, 105, 221, 323, 346, 65, 259, 254, 400, 345, 400, 177, 321, 167, 317, 352, 19, 198, 324, 156, 164, 104, 66, 319, 216, 327, 200, 142, 166, 226, 43, 300, 23, 44, 52, 41, 230, 395, 256, 338, 309, 348, 323, 159, 389, 265, 170, 310, 229, 350, 311, 173, 32, 352, 274, 39, 5, 363, 7, 398, 318, 220, 35, 302, 149, 134, 261, 132, 281, 146, 322, 5, 352, 85, 111, 201, 359, 88, 47, 277, 11, 43, 125, 195, 202, 68, 55, 73, 62, 175, 236, 203, 248, 300, 393, 121, 398, 181, 127, 247, 227, 128, 64, 214, 290, 231, 93, 385, 242, 179, 137, 301, 365, 376, 312, 37, 359, 361, 43, 38, 57, 195, 170, 77, 90, 311, 383, 210, 19, 164, 104, 199, 96, 104, 53, 361, 306, 256, 115, 234, 316, 226, 392, 209, 395, 17, 371, 280, 86, 83, 301, 141, 176, 186, 318, 348, 242, 42, 93, 90, 87, 180, 216, 126, 193, 334, 135, 334, 183, 321, 319, 256, 196, 306, 111, 107, 118, 19, 146, 70, 365, 240, 110, 311, 298, 170, 395, 393, 186, 213, 131, 93, 338, 148, 208, 179, 253, 380, 55, 21, 149, 163, 313, 330, 206, 227, 65, 270, 24, 254, 84, 277, 26, 198, 242, 374, 400, 385, 182, 3, 378, 71, 23, 9, 266, 27, 374, 235, 189, 17, 282, 56, 289, 262, 91, 89, 161, 323, 73, 297, 114, 39, 161, 237, 364])
scrabler.append([390, 208, 75, 226, 177, 365, 205, 169, 355, 145, 186, 349, 153, 361, 346, 329, 7, 329, 227, 157, 374, 253, 383, 56, 136, 220, 225, 202, 397, 211, 6, 79, 351, 208, 148, 17, 298, 35, 112, 4, 283, 75, 382, 250, 228, 341, 188, 367, 123, 128, 291, 165, 186, 85, 176, 334, 202, 39, 179, 197, 191, 343, 7, 219, 246, 376, 237, 273, 155, 115, 278, 148, 226, 388, 338, 397, 151, 217, 295, 321, 195, 230, 137, 297, 101, 384, 101, 57, 299, 292, 84, 59, 216, 65, 314, 352, 333, 295, 170, 55, 201, 320, 149, 112, 285, 382, 85, 191, 355, 181, 377, 83, 298, 26, 273, 111, 311, 3, 35, 146, 24, 136, 390, 16, 319, 31, 124, 284, 278, 240, 76, 141, 22, 50, 277, 276, 352, 306, 348, 190, 388, 107, 4, 241, 391, 175, 171, 6, 149, 116, 285, 385, 10, 98, 243, 69, 45, 270, 118, 73, 380, 397, 87, 3, 69, 295, 196, 213, 289, 101, 379, 232, 27, 150, 68, 96, 200, 146, 359, 88, 93, 335, 23, 335, 32, 4, 254, 225, 32, 40, 153, 357, 336, 46, 77, 213, 292, 179, 217, 143, 5, 372, 30, 85, 304, 161, 9, 196, 212, 225, 393, 27, 313, 20, 366, 19, 81, 223, 287, 279, 293, 257, 110, 60, 195, 5, 353, 8, 169, 238, 335, 195, 353, 250, 227, 335, 252, 356, 164, 49, 102, 279, 152, 193, 78, 148, 155, 29, 40, 198, 88, 381, 385, 73, 28, 301, 264, 217, 7, 109, 289, 388, 324, 306, 308, 310, 215, 194, 311, 88, 200, 75, 283, 147, 54, 362, 279, 2, 349, 198, 366, 82, 74, 108, 299, 349, 226, 45, 398, 141, 275, 131, 276, 330, 173, 314, 16, 200, 320, 82, 177, 301, 212, 125, 224, 57, 169, 217, 379, 153, 201, 154, 226, 276, 400, 12, 246, 2, 293, 103, 20, 201, 306, 157, 210, 389, 173, 319, 246, 77, 131, 358, 306, 67, 359, 88, 390, 169, 386, 292, 156, 39, 357, 100, 277, 13, 194, 263, 358, 288, 59, 340, 128, 114, 350, 134, 383, 196, 285, 164, 165, 306, 129, 162, 331, 390, 390, 314, 187, 11, 129, 39, 158, 189, 346, 303, 195, 372, 23, 117, 364, 271, 172, 175, 22, 271, 180, 316, 226, 31, 74, 144, 214, 96, 385, 1, 48, 1, 371, 283])
scrabler.append([128, 251, 90, 45, 124, 308, 325, 38, 85, 319, 31, 25, 180, 337, 94, 5, 145, 23, 112, 348, 102, 279, 118, 367, 342, 78, 137, 159, 80, 293, 16, 133, 99, 75, 357, 286, 222, 60, 289, 342, 125, 204, 282, 280, 174, 13, 117, 144, 116, 323, 350, 253, 138, 367, 389, 307, 23, 337, 147, 88, 284, 157, 342, 218, 10, 248, 195, 381, 363, 134, 89, 197, 226, 196, 228, 323, 88, 91, 208, 154, 189, 265, 13, 46, 348, 329, 199, 288, 215, 318, 12, 355, 315, 184, 48, 280, 162, 21, 53, 309, 133, 271, 194, 107, 90, 315, 214, 201, 86, 137, 45, 161, 395, 341, 2, 268, 59, 400, 95, 7, 337, 342, 123, 346, 245, 112, 135, 58, 194, 227, 6, 113, 54, 4, 248, 23, 20, 282, 317, 229, 260, 322, 351, 236, 228, 130, 105, 252, 51, 329, 316, 247, 56, 101, 67, 215, 344, 29, 203, 276, 19, 232, 252, 231, 16, 93, 317, 23, 358, 87, 154, 161, 128, 155, 216, 162, 259, 343, 348, 326, 88, 325, 316, 82, 361, 90, 345, 206, 395, 262, 296, 257, 150, 39, 294, 104, 115, 309, 282, 16, 195, 71, 369, 57, 363, 25, 218, 194, 323, 363, 294, 397, 249, 274, 288, 28, 42, 394, 177, 281, 362, 142, 301, 46, 375, 5, 165, 207, 199, 211, 75, 38, 42, 321, 160, 280, 300, 355, 380, 120, 39, 2, 209, 79, 15, 344, 135, 233, 158, 28, 345, 333, 357, 329, 72, 310, 385, 231, 336, 280, 66, 161, 298, 208, 48, 199, 272, 162, 12, 3, 89, 232, 30, 371, 111, 159, 313, 138, 335, 314, 185, 52, 295, 174, 225, 298, 189, 250, 114, 281, 18, 244, 332, 102, 31, 212, 124, 264, 152, 332, 107, 153, 287, 301, 264, 322, 108, 194, 170, 130, 192, 107, 294, 15, 359, 104, 219, 379, 307, 353, 124, 203, 27, 189, 80, 197, 200, 196, 322, 179, 136, 325, 61, 47, 1, 391, 43, 274, 73, 10, 357, 107, 349, 137, 163, 162, 156, 254, 283, 285, 334, 291, 6, 102, 300, 229, 283, 78, 54, 397, 90, 138, 169, 174, 14, 366, 4, 98, 97, 160, 306, 384, 210, 254, 93, 183, 237, 317, 70, 285, 165, 180, 10, 238, 132, 282, 2, 53, 283, 105, 255, 164, 46, 289, 400, 170, 32, 277, 72, 138])
scrabler.append([174, 99, 323, 153, 159, 122, 183, 227, 380, 391, 52, 297, 346, 357, 115, 176, 177, 200, 218, 196, 367, 116, 299, 20, 345, 392, 52, 311, 271, 32, 103, 130, 153, 396, 308, 143, 86, 346, 207, 55, 279, 275, 242, 165, 151, 227, 45, 331, 24, 138, 369, 345, 26, 93, 65, 204, 122, 360, 322, 297, 271, 55, 27, 321, 40, 64, 302, 191, 16, 4, 248, 133, 81, 273, 136, 153, 177, 317, 0, 15, 274, 151, 262, 223, 141, 326, 38, 137, 138, 40, 52, 181, 385, 320, 31, 187, 344, 58, 116, 309, 334, 262, 115, 364, 91, 228, 21, 256, 230, 34, 220, 357, 194, 88, 304, 1, 256, 8, 69, 0, 375, 321, 133, 220, 120, 325, 327, 214, 336, 377, 312, 365, 145, 299, 340, 209, 45, 101, 172, 174, 236, 269, 206, 3, 21, 234, 223, 343, 23, 143, 84, 197, 64, 384, 106, 151, 271, 335, 154, 333, 366, 316, 331, 168, 22, 30, 211, 382, 95, 145, 314, 328, 391, 291, 298, 63, 58, 280, 23, 345, 118, 237, 117, 297, 205, 361, 388, 156, 171, 183, 345, 188, 191, 258, 354, 382, 83, 390, 123, 281, 45, 141, 101, 294, 49, 166, 374, 257, 95, 136, 228, 159, 259, 336, 302, 92, 300, 185, 346, 19, 131, 259, 88, 234, 59, 363, 178, 29, 262, 360, 84, 173, 158, 163, 262, 168, 124, 8, 45, 389, 372, 369, 377, 44, 279, 162, 8, 72, 267, 242, 138, 230, 361, 318, 348, 373, 79, 110, 397, 233, 204, 201, 136, 344, 356, 160, 344, 14, 348, 150, 226, 386, 188, 251, 390, 222, 50, 269, 266, 24, 186, 71, 201, 380, 326, 279, 297, 32, 121, 164, 173, 272, 232, 66, 106, 387, 370, 105, 349, 198, 317, 157, 242, 372, 11, 173, 128, 44, 237, 229, 326, 25, 354, 53, 111, 199, 266, 329, 357, 175, 91, 191, 398, 272, 317, 96, 288, 129, 140, 12, 271, 176, 278, 60, 343, 138, 390, 230, 261, 211, 113, 125, 228, 343, 56, 373, 66, 289, 134, 300, 86, 327, 3, 285, 69, 328, 29, 316, 126, 133, 192, 122, 164, 150, 209, 324, 67, 382, 110, 291, 330, 287, 11, 128, 110, 134, 128, 161, 156, 331, 303, 162, 10, 78, 46, 211, 42, 311, 269, 228, 359, 331, 218, 344, 67, 85, 275, 316, 336, 62])
scrabler.append([189, 125, 17, 185, 359, 63, 73, 89, 381, 261, 5, 220, 74, 262, 333, 377, 131, 193, 76, 176, 305, 83, 65, 198, 234, 144, 33, 84, 392, 276, 192, 157, 350, 70, 14, 184, 100, 194, 191, 143, 273, 40, 299, 92, 234, 333, 289, 101, 96, 349, 9, 62, 362, 1, 344, 359, 182, 323, 40, 235, 147, 222, 87, 94, 10, 76, 201, 275, 249, 126, 243, 19, 367, 26, 97, 200, 234, 168, 279, 131, 29, 200, 198, 39, 303, 93, 107, 136, 364, 65, 13, 337, 15, 166, 174, 51, 42, 399, 244, 102, 87, 158, 155, 322, 63, 270, 346, 320, 221, 374, 68, 392, 206, 374, 354, 27, 147, 136, 227, 294, 56, 131, 17, 305, 269, 117, 174, 19, 54, 51, 75, 141, 399, 279, 183, 254, 44, 77, 78, 256, 125, 162, 214, 71, 148, 367, 310, 191, 153, 154, 230, 252, 388, 167, 160, 239, 390, 233, 382, 20, 232, 63, 365, 164, 84, 122, 91, 309, 162, 353, 300, 374, 144, 57, 18, 255, 169, 27, 184, 177, 93, 228, 62, 90, 173, 64, 54, 7, 276, 57, 393, 39, 159, 16, 278, 239, 116, 26, 199, 286, 152, 53, 10, 121, 211, 169, 255, 7, 249, 397, 285, 331, 196, 110, 127, 304, 169, 314, 300, 328, 276, 54, 235, 96, 77, 65, 347, 245, 377, 208, 162, 231, 222, 228, 173, 364, 31, 182, 369, 311, 16, 250, 49, 372, 92, 373, 202, 77, 55, 239, 34, 93, 336, 148, 260, 80, 9, 377, 343, 84, 72, 241, 13, 71, 101, 215, 34, 101, 95, 72, 339, 303, 18, 189, 30, 307, 27, 226, 356, 385, 88, 159, 386, 390, 4, 361, 119, 330, 353, 350, 39, 77, 72, 40, 181, 232, 309, 110, 235, 268, 140, 287, 333, 178, 36, 132, 231, 123, 182, 36, 319, 321, 248, 136, 108, 233, 235, 366, 11, 304, 84, 397, 81, 359, 104, 161, 222, 231, 0, 65, 330, 306, 351, 60, 350, 126, 308, 354, 9, 177, 220, 70, 231, 92, 0, 189, 193, 357, 48, 169, 140, 61, 389, 115, 259, 54, 63, 202, 68, 241, 305, 269, 182, 207, 158, 242, 294, 147, 248, 284, 360, 319, 268, 246, 59, 196, 236, 168, 14, 392, 250, 214, 361, 159, 281, 241, 25, 271, 106, 229, 220, 261, 86, 243, 233, 96, 75, 366, 148, 212])
scrabler.append([99, 264, 380, 270, 67, 134, 59, 368, 184, 347, 198, 300, 172, 119, 196, 53, 162, 40, 124, 30, 265, 72, 177, 192, 75, 202, 282, 266, 283, 230, 247, 51, 281, 182, 341, 258, 48, 71, 280, 226, 289, 130, 398, 189, 124, 152, 119, 51, 88, 137, 117, 264, 347, 39, 389, 342, 329, 157, 317, 168, 344, 103, 133, 132, 153, 187, 43, 235, 133, 70, 307, 86, 362, 65, 206, 110, 70, 128, 302, 61, 130, 21, 211, 70, 43, 101, 381, 305, 219, 53, 311, 20, 276, 18, 40, 17, 15, 190, 18, 148, 245, 289, 308, 99, 379, 105, 114, 366, 333, 326, 152, 146, 335, 279, 386, 298, 120, 114, 157, 334, 111, 172, 360, 202, 35, 175, 335, 64, 157, 191, 284, 33, 76, 64, 236, 377, 267, 17, 249, 314, 252, 19, 28, 63, 337, 207, 367, 176, 53, 383, 373, 103, 298, 274, 224, 264, 90, 205, 325, 123, 381, 137, 365, 70, 322, 137, 370, 208, 257, 200, 102, 131, 30, 127, 313, 227, 380, 293, 84, 200, 181, 93, 324, 22, 245, 182, 55, 4, 167, 204, 270, 293, 105, 19, 199, 4, 254, 309, 26, 225, 391, 52, 355, 195, 101, 183, 360, 89, 38, 348, 107, 70, 149, 334, 225, 70, 126, 128, 8, 60, 279, 275, 260, 330, 386, 378, 288, 395, 364, 310, 1, 129, 67, 338, 332, 154, 146, 128, 66, 294, 79, 133, 235, 365, 269, 144, 272, 319, 298, 306, 74, 78, 349, 259, 126, 383, 144, 101, 198, 345, 76, 292, 219, 28, 365, 220, 189, 284, 366, 274, 309, 284, 38, 89, 370, 326, 52, 146, 27, 99, 71, 103, 204, 223, 111, 383, 162, 251, 331, 156, 232, 349, 232, 354, 167, 67, 249, 267, 343, 377, 177, 317, 225, 264, 163, 206, 269, 251, 95, 73, 153, 6, 209, 130, 396, 380, 22, 43, 36, 141, 373, 30, 253, 174, 29, 314, 67, 225, 294, 48, 95, 282, 72, 63, 177, 277, 385, 214, 331, 180, 400, 169, 156, 289, 322, 398, 59, 350, 341, 190, 293, 350, 164, 382, 350, 65, 192, 291, 166, 229, 83, 326, 12, 257, 300, 385, 0, 353, 329, 284, 5, 60, 39, 150, 391, 333, 245, 139, 24, 219, 96, 298, 58, 162, 67, 250, 169, 336, 53, 208, 33, 269, 253, 220, 243, 57, 179, 103, 213, 111])
scrabler.append([160, 199, 192, 5, 139, 48, 237, 105, 365, 345, 197, 227, 215, 268, 394, 49, 271, 259, 111, 11, 315, 27, 305, 17, 39, 213, 223, 204, 313, 247, 239, 269, 317, 225, 153, 110, 380, 60, 391, 5, 381, 148, 292, 265, 191, 372, 115, 300, 125, 41, 335, 168, 100, 308, 67, 140, 326, 215, 357, 378, 326, 135, 318, 390, 180, 264, 146, 369, 194, 156, 212, 77, 366, 325, 190, 242, 143, 353, 36, 216, 22, 240, 273, 342, 63, 43, 230, 235, 239, 273, 315, 98, 393, 216, 67, 241, 103, 245, 111, 316, 106, 244, 237, 271, 301, 120, 142, 363, 170, 278, 26, 272, 272, 209, 36, 153, 185, 350, 105, 29, 188, 230, 92, 183, 269, 363, 100, 60, 4, 340, 255, 137, 237, 227, 246, 342, 270, 357, 151, 387, 68, 41, 339, 392, 231, 131, 46, 0, 266, 301, 309, 28, 300, 109, 311, 205, 22, 226, 29, 354, 300, 257, 386, 135, 185, 309, 392, 54, 356, 303, 147, 183, 399, 327, 336, 115, 282, 213, 301, 253, 315, 380, 177, 123, 387, 70, 356, 112, 249, 5, 303, 112, 206, 355, 116, 376, 397, 5, 354, 228, 275, 309, 104, 21, 352, 106, 96, 346, 54, 101, 172, 277, 135, 60, 263, 329, 95, 266, 89, 356, 343, 342, 332, 55, 346, 240, 112, 278, 197, 46, 93, 228, 264, 45, 178, 350, 358, 290, 80, 214, 100, 103, 239, 123, 399, 96, 124, 121, 393, 54, 125, 68, 157, 96, 5, 68, 257, 382, 241, 252, 21, 352, 187, 339, 178, 73, 258, 218, 81, 110, 152, 377, 55, 352, 383, 283, 376, 264, 7, 216, 327, 110, 127, 131, 61, 299, 267, 368, 177, 23, 176, 292, 143, 160, 369, 56, 225, 295, 127, 390, 212, 322, 388, 324, 389, 370, 179, 315, 32, 262, 287, 44, 279, 4, 399, 184, 304, 18, 370, 248, 96, 177, 192, 86, 148, 155, 160, 102, 174, 378, 318, 371, 145, 328, 72, 222, 259, 25, 325, 156, 27, 160, 50, 15, 161, 4, 142, 266, 165, 203, 53, 400, 372, 276, 351, 151, 333, 132, 79, 98, 20, 193, 231, 177, 242, 94, 271, 254, 157, 4, 219, 130, 178, 99, 367, 49, 86, 11, 330, 278, 282, 147, 110, 298, 111, 156, 266, 263, 288, 218, 393, 354, 344, 108, 298, 268, 8, 335, 306, 134])
scrabler.append([220, 268, 59, 292, 43, 54, 348, 211, 51, 364, 40, 149, 62, 80, 198, 225, 129, 52, 222, 29, 105, 95, 242, 91, 374, 300, 392, 141, 116, 96, 70, 155, 377, 320, 60, 202, 396, 208, 132, 23, 316, 383, 157, 259, 25, 287, 338, 33, 139, 359, 115, 208, 77, 373, 55, 63, 171, 232, 334, 79, 345, 138, 33, 266, 41, 298, 292, 147, 126, 155, 364, 141, 385, 58, 349, 246, 33, 340, 398, 107, 197, 104, 56, 302, 2, 359, 183, 261, 312, 363, 260, 214, 6, 293, 98, 5, 380, 326, 178, 304, 361, 147, 280, 287, 378, 133, 146, 33, 375, 196, 294, 39, 29, 70, 80, 90, 89, 276, 168, 152, 152, 303, 147, 329, 334, 19, 82, 275, 48, 298, 368, 231, 284, 4, 285, 92, 32, 378, 352, 113, 356, 198, 382, 271, 389, 131, 117, 193, 279, 225, 347, 287, 216, 41, 46, 61, 218, 394, 323, 394, 233, 265, 24, 132, 396, 255, 195, 114, 331, 340, 288, 285, 202, 145, 319, 46, 145, 85, 186, 216, 254, 173, 391, 93, 353, 204, 352, 37, 323, 72, 347, 358, 44, 162, 195, 128, 264, 264, 300, 215, 69, 388, 140, 339, 297, 6, 122, 3, 209, 168, 109, 349, 273, 322, 151, 172, 8, 333, 329, 369, 377, 384, 393, 5, 367, 106, 165, 222, 279, 86, 55, 400, 353, 363, 201, 153, 148, 269, 197, 215, 304, 210, 241, 300, 119, 47, 261, 314, 196, 107, 283, 303, 180, 275, 128, 85, 384, 264, 66, 218, 179, 274, 319, 118, 199, 146, 348, 358, 135, 389, 11, 119, 95, 319, 322, 277, 65, 281, 363, 369, 144, 247, 312, 212, 50, 222, 24, 84, 190, 126, 69, 256, 375, 140, 76, 68, 55, 296, 313, 282, 384, 106, 307, 315, 239, 247, 113, 298, 159, 255, 198, 333, 310, 230, 57, 182, 383, 329, 310, 35, 352, 239, 106, 385, 39, 301, 154, 307, 378, 103, 268, 370, 308, 80, 322, 59, 201, 7, 19, 274, 193, 234, 378, 162, 113, 181, 116, 29, 354, 137, 251, 230, 232, 237, 76, 84, 197, 6, 293, 72, 243, 396, 202, 162, 398, 28, 64, 27, 366, 302, 37, 171, 208, 264, 57, 90, 353, 186, 17, 39, 134, 392, 39, 37, 8, 88, 157, 231, 33, 13, 234, 335, 226, 335, 298, 206, 52, 124, 305, 0])
scrabler.append([98, 262, 181, 131, 114, 209, 133, 359, 269, 31, 135, 225, 172, 64, 238, 363, 200, 356, 380, 113, 397, 154, 29, 25, 270, 393, 99, 217, 104, 104, 281, 265, 197, 141, 112, 134, 400, 5, 343, 16, 47, 347, 291, 324, 322, 223, 76, 196, 396, 374, 121, 265, 244, 10, 379, 63, 295, 92, 95, 353, 222, 330, 156, 349, 363, 167, 137, 33, 202, 198, 128, 10, 207, 271, 176, 180, 363, 225, 343, 378, 107, 101, 176, 236, 154, 30, 371, 42, 351, 181, 45, 137, 29, 326, 76, 208, 176, 108, 378, 41, 283, 36, 400, 345, 106, 198, 179, 96, 52, 98, 319, 300, 77, 31, 110, 9, 237, 319, 278, 331, 136, 227, 196, 371, 256, 179, 55, 35, 270, 87, 130, 230, 112, 208, 26, 327, 127, 259, 150, 65, 72, 364, 356, 119, 394, 230, 394, 285, 31, 9, 205, 373, 15, 18, 289, 183, 89, 226, 274, 32, 334, 354, 390, 313, 173, 236, 169, 239, 139, 213, 176, 142, 307, 110, 224, 204, 329, 162, 340, 156, 358, 65, 211, 102, 333, 233, 128, 179, 357, 163, 278, 68, 107, 126, 370, 389, 400, 190, 155, 167, 87, 85, 349, 105, 389, 176, 391, 219, 68, 162, 143, 41, 323, 34, 19, 172, 176, 344, 198, 199, 31, 371, 234, 299, 229, 222, 367, 341, 227, 203, 328, 230, 65, 387, 188, 359, 54, 191, 44, 179, 360, 263, 211, 291, 352, 310, 114, 19, 72, 80, 125, 255, 217, 196, 375, 331, 157, 311, 200, 36, 122, 288, 187, 338, 254, 347, 251, 150, 332, 86, 376, 194, 348, 307, 288, 109, 273, 81, 30, 276, 141, 50, 162, 140, 277, 345, 237, 220, 85, 150, 333, 244, 314, 204, 169, 104, 366, 218, 268, 264, 330, 384, 235, 214, 386, 162, 122, 55, 113, 157, 293, 110, 266, 391, 11, 164, 382, 362, 55, 199, 223, 120, 97, 237, 164, 248, 40, 210, 307, 31, 223, 283, 91, 384, 168, 262, 40, 146, 68, 276, 170, 161, 0, 379, 66, 192, 345, 89, 396, 165, 339, 69, 19, 293, 264, 191, 191, 181, 293, 391, 9, 148, 237, 81, 88, 307, 17, 123, 264, 176, 35, 189, 23, 18, 70, 36, 353, 166, 181, 43, 131, 226, 207, 53, 248, 376, 292, 367, 206, 127, 43, 340, 396, 226, 41, 57, 121, 129, 12, 62])
scrabler.append([207, 124, 167, 162, 149, 300, 324, 255, 220, 373, 93, 25, 239, 50, 251, 142, 131, 355, 102, 248, 78, 228, 157, 212, 36, 96, 54, 320, 14, 115, 88, 195, 87, 150, 387, 70, 35, 283, 135, 285, 92, 145, 98, 352, 98, 153, 366, 366, 63, 351, 91, 399, 12, 168, 315, 114, 230, 29, 340, 248, 300, 136, 189, 188, 314, 4, 41, 335, 46, 28, 29, 80, 3, 207, 51, 107, 170, 5, 158, 353, 304, 139, 122, 70, 146, 90, 42, 159, 13, 35, 113, 155, 400, 82, 1, 35, 387, 191, 1, 32, 68, 265, 96, 93, 152, 185, 360, 9, 386, 400, 22, 87, 212, 351, 56, 308, 355, 61, 339, 160, 271, 346, 368, 185, 387, 104, 298, 96, 98, 279, 141, 169, 321, 245, 156, 176, 238, 140, 285, 80, 207, 378, 6, 327, 171, 230, 216, 62, 210, 68, 123, 169, 110, 83, 21, 180, 340, 283, 278, 57, 336, 372, 377, 60, 12, 53, 213, 322, 54, 223, 87, 49, 235, 106, 203, 359, 105, 29, 183, 260, 228, 280, 45, 58, 35, 0, 238, 314, 28, 238, 144, 137, 205, 72, 194, 139, 220, 298, 195, 168, 8, 49, 361, 306, 226, 175, 395, 61, 186, 328, 377, 233, 12, 180, 138, 177, 319, 388, 369, 283, 209, 242, 304, 319, 267, 27, 140, 381, 19, 293, 348, 190, 235, 192, 373, 261, 228, 364, 118, 34, 315, 110, 36, 320, 255, 207, 88, 52, 131, 294, 298, 100, 131, 165, 241, 197, 27, 82, 337, 111, 327, 369, 263, 204, 359, 202, 54, 371, 162, 124, 158, 145, 349, 216, 142, 267, 209, 296, 2, 331, 2, 163, 82, 337, 254, 55, 349, 332, 242, 248, 299, 30, 9, 93, 8, 276, 179, 165, 215, 243, 113, 173, 216, 216, 49, 369, 393, 17, 143, 379, 230, 149, 316, 7, 306, 307, 173, 54, 253, 101, 53, 363, 316, 194, 338, 312, 70, 249, 104, 328, 45, 149, 34, 58, 162, 336, 146, 249, 324, 167, 398, 389, 35, 372, 148, 369, 279, 151, 265, 254, 332, 305, 46, 352, 212, 60, 328, 20, 362, 365, 238, 243, 384, 180, 278, 58, 64, 285, 303, 284, 375, 315, 243, 113, 170, 96, 392, 328, 370, 121, 211, 139, 241, 353, 143, 316, 119, 40, 305, 299, 328, 67, 96, 127, 292, 202, 266, 255, 197, 354])
scrabler.append([127, 319, 251, 256, 144, 255, 42, 217, 354, 140, 399, 222, 84, 128, 205, 155, 98, 236, 133, 121, 355, 71, 8, 270, 175, 201, 123, 210, 200, 247, 208, 306, 153, 163, 188, 228, 346, 357, 202, 392, 105, 82, 90, 308, 16, 318, 323, 205, 316, 400, 184, 48, 205, 329, 50, 187, 264, 112, 0, 206, 27, 164, 143, 178, 33, 317, 277, 307, 66, 132, 179, 333, 213, 337, 367, 70, 285, 306, 355, 397, 106, 304, 48, 322, 386, 289, 292, 400, 311, 125, 265, 9, 357, 106, 248, 31, 64, 86, 176, 270, 343, 101, 382, 278, 347, 298, 267, 339, 389, 55, 333, 140, 160, 295, 9, 9, 307, 108, 313, 352, 4, 196, 41, 191, 261, 142, 7, 146, 14, 131, 296, 348, 358, 142, 248, 148, 213, 287, 357, 165, 269, 159, 58, 154, 132, 264, 257, 273, 267, 40, 353, 272, 331, 272, 245, 236, 118, 283, 163, 162, 278, 357, 87, 180, 236, 248, 15, 133, 339, 257, 78, 136, 148, 162, 161, 18, 282, 192, 226, 338, 59, 242, 6, 333, 100, 326, 143, 238, 210, 68, 371, 143, 364, 68, 135, 387, 12, 370, 2, 380, 385, 32, 399, 219, 137, 145, 91, 315, 158, 200, 279, 235, 120, 342, 230, 352, 207, 230, 83, 267, 201, 270, 13, 384, 175, 71, 175, 6, 139, 277, 41, 94, 331, 357, 20, 302, 209, 58, 124, 27, 33, 76, 377, 143, 187, 43, 375, 31, 292, 328, 306, 281, 207, 368, 64, 38, 297, 62, 132, 17, 89, 71, 292, 280, 354, 176, 204, 45, 249, 159, 164, 209, 219, 359, 74, 241, 318, 344, 136, 7, 268, 213, 75, 234, 192, 156, 220, 317, 237, 76, 327, 265, 129, 114, 369, 374, 12, 392, 16, 175, 49, 180, 113, 110, 33, 298, 343, 120, 346, 69, 284, 394, 308, 96, 151, 0, 368, 70, 245, 123, 334, 25, 299, 144, 84, 107, 207, 10, 189, 292, 390, 398, 377, 62, 143, 277, 43, 8, 202, 14, 4, 323, 254, 285, 278, 184, 237, 397, 219, 263, 367, 337, 299, 400, 181, 155, 273, 282, 223, 345, 106, 143, 154, 138, 394, 97, 202, 394, 49, 121, 71, 130, 72, 158, 362, 85, 62, 312, 340, 340, 387, 386, 385, 390, 56, 379, 141, 261, 288, 161, 249, 209, 396, 73, 375, 304, 251, 110, 219, 385])
scrabler.append([274, 231, 383, 299, 284, 34, 358, 113, 309, 147, 174, 180, 264, 389, 34, 249, 149, 198, 395, 153, 190, 53, 202, 398, 26, 317, 161, 398, 84, 34, 80, 175, 62, 131, 120, 175, 320, 190, 201, 141, 371, 268, 73, 144, 39, 175, 301, 261, 362, 217, 79, 77, 11, 395, 242, 77, 182, 163, 36, 398, 262, 155, 108, 85, 320, 14, 13, 274, 338, 87, 279, 175, 198, 309, 310, 210, 77, 29, 106, 203, 358, 52, 356, 176, 349, 243, 11, 346, 124, 374, 17, 332, 171, 199, 313, 394, 377, 221, 274, 209, 89, 210, 59, 7, 167, 13, 277, 323, 179, 272, 60, 78, 253, 121, 149, 341, 107, 378, 169, 51, 368, 136, 309, 99, 35, 323, 228, 78, 391, 259, 241, 86, 113, 49, 10, 291, 257, 10, 195, 196, 315, 390, 29, 109, 389, 177, 140, 190, 375, 271, 96, 109, 138, 181, 290, 327, 121, 214, 353, 131, 330, 130, 203, 308, 262, 161, 68, 148, 261, 295, 113, 69, 125, 42, 368, 235, 261, 101, 160, 400, 157, 400, 102, 129, 134, 395, 214, 224, 376, 6, 53, 192, 136, 176, 68, 42, 348, 210, 233, 360, 84, 296, 359, 309, 339, 184, 128, 70, 10, 312, 104, 117, 193, 287, 186, 96, 223, 349, 148, 256, 59, 131, 369, 178, 225, 154, 393, 195, 144, 347, 209, 382, 397, 279, 282, 61, 92, 168, 96, 285, 174, 54, 159, 111, 197, 349, 39, 244, 214, 250, 309, 193, 371, 19, 342, 337, 195, 213, 285, 192, 1, 147, 75, 227, 285, 73, 153, 234, 251, 341, 185, 2, 249, 177, 383, 257, 299, 215, 287, 39, 192, 319, 177, 193, 232, 103, 316, 208, 252, 102, 98, 328, 181, 137, 199, 100, 246, 349, 173, 379, 272, 230, 372, 69, 325, 96, 169, 108, 296, 197, 191, 398, 352, 176, 29, 368, 192, 93, 108, 379, 210, 363, 332, 174, 38, 307, 104, 130, 85, 175, 51, 34, 309, 61, 137, 395, 375, 312, 188, 122, 300, 272, 84, 371, 335, 231, 101, 234, 51, 317, 125, 310, 361, 240, 182, 246, 196, 329, 305, 321, 67, 393, 253, 334, 71, 292, 6, 217, 145, 370, 309, 322, 86, 123, 399, 94, 364, 263, 109, 55, 46, 90, 260, 372, 319, 62, 83, 23, 150, 340, 26, 104, 387, 366, 71, 307, 238, 27, 14, 106])
scrabler.append([120, 197, 6, 86, 192, 145, 272, 140, 243, 314, 67, 277, 348, 329, 154, 378, 84, 386, 385, 265, 213, 218, 121, 92, 236, 35, 342, 359, 333, 83, 169, 16, 277, 354, 350, 222, 169, 56, 334, 23, 162, 44, 39, 237, 16, 182, 303, 330, 304, 305, 181, 273, 79, 224, 153, 285, 236, 378, 230, 230, 79, 378, 10, 1, 303, 91, 170, 324, 5, 111, 290, 343, 103, 318, 208, 343, 72, 248, 357, 346, 185, 299, 240, 59, 204, 251, 356, 346, 27, 95, 169, 271, 400, 163, 107, 329, 79, 225, 344, 57, 193, 80, 282, 139, 357, 262, 178, 217, 332, 215, 133, 278, 283, 115, 177, 190, 381, 142, 269, 350, 312, 226, 377, 70, 329, 36, 217, 178, 397, 137, 265, 378, 15, 108, 306, 261, 276, 4, 160, 185, 34, 234, 218, 192, 166, 315, 289, 196, 296, 386, 359, 356, 224, 57, 316, 334, 102, 12, 380, 248, 395, 392, 196, 305, 324, 325, 246, 341, 370, 202, 164, 131, 395, 316, 364, 39, 381, 240, 125, 159, 297, 165, 252, 168, 357, 376, 37, 112, 281, 58, 160, 274, 240, 107, 254, 36, 392, 194, 75, 369, 217, 256, 163, 345, 91, 62, 100, 326, 396, 133, 64, 285, 64, 169, 107, 211, 355, 56, 342, 378, 84, 264, 269, 234, 204, 53, 144, 255, 91, 51, 323, 363, 16, 303, 100, 317, 199, 27, 350, 288, 153, 274, 148, 86, 220, 396, 78, 144, 256, 57, 225, 123, 240, 264, 277, 197, 364, 172, 262, 171, 317, 235, 373, 75, 294, 267, 235, 114, 91, 256, 215, 86, 147, 168, 176, 121, 170, 92, 177, 93, 303, 84, 378, 225, 235, 21, 19, 270, 306, 376, 66, 261, 356, 5, 91, 303, 263, 278, 33, 20, 71, 292, 228, 132, 128, 388, 364, 64, 367, 34, 22, 325, 83, 382, 291, 237, 189, 1, 90, 115, 356, 147, 111, 66, 273, 232, 340, 175, 134, 68, 159, 396, 219, 381, 29, 128, 7, 149, 242, 399, 258, 96, 287, 187, 188, 108, 229, 16, 183, 58, 349, 360, 120, 52, 146, 285, 306, 58, 340, 381, 335, 366, 19, 368, 22, 338, 110, 398, 296, 179, 302, 396, 15, 8, 210, 384, 324, 286, 371, 289, 45, 143, 276, 155, 369, 205, 348, 118, 191, 400, 256, 170, 69, 30, 81, 274, 371, 73, 285, 224])
scrabler.append([151, 289, 376, 114, 118, 200, 389, 69, 297, 174, 174, 82, 30, 399, 215, 189, 157, 163, 210, 61, 151, 175, 34, 180, 211, 106, 188, 192, 176, 43, 290, 320, 107, 47, 123, 54, 33, 328, 363, 295, 368, 362, 80, 288, 254, 285, 164, 36, 262, 214, 91, 224, 369, 241, 149, 308, 257, 307, 112, 82, 308, 80, 195, 277, 135, 100, 19, 364, 353, 178, 321, 390, 253, 315, 55, 197, 278, 257, 367, 312, 71, 341, 346, 44, 91, 241, 85, 356, 387, 124, 375, 3, 210, 204, 276, 182, 25, 250, 95, 266, 84, 187, 133, 204, 212, 92, 96, 181, 87, 197, 50, 378, 285, 64, 107, 3, 303, 345, 327, 388, 121, 56, 276, 201, 51, 379, 102, 129, 175, 355, 22, 109, 390, 18, 52, 391, 310, 263, 66, 182, 179, 45, 29, 185, 156, 343, 258, 293, 77, 328, 315, 398, 206, 273, 155, 307, 261, 388, 107, 118, 148, 21, 67, 222, 150, 195, 127, 167, 160, 281, 78, 192, 302, 157, 339, 307, 335, 295, 286, 240, 160, 99, 388, 298, 131, 325, 25, 0, 96, 86, 182, 326, 156, 94, 386, 247, 350, 41, 117, 251, 379, 139, 38, 17, 385, 322, 288, 229, 387, 302, 45, 307, 142, 165, 6, 340, 343, 34, 330, 230, 320, 256, 356, 336, 363, 174, 173, 147, 89, 318, 250, 132, 150, 209, 287, 235, 206, 288, 363, 122, 242, 262, 146, 241, 222, 113, 144, 301, 167, 46, 382, 260, 180, 304, 122, 10, 213, 366, 199, 395, 315, 300, 208, 219, 128, 202, 5, 242, 168, 95, 205, 366, 220, 223, 381, 140, 181, 380, 263, 49, 183, 262, 268, 248, 322, 212, 229, 107, 194, 45, 62, 289, 309, 285, 141, 257, 189, 235, 247, 254, 350, 78, 123, 34, 166, 118, 300, 123, 325, 121, 52, 232, 89, 12, 22, 302, 32, 379, 162, 208, 206, 197, 116, 213, 243, 195, 248, 157, 244, 21, 379, 206, 340, 37, 395, 227, 166, 359, 302, 230, 362, 265, 122, 103, 20, 37, 40, 360, 301, 143, 151, 193, 223, 64, 93, 165, 381, 225, 48, 204, 208, 11, 205, 367, 161, 303, 230, 364, 169, 95, 68, 74, 19, 336, 200, 150, 79, 345, 277, 366, 358, 242, 380, 17, 376, 29, 188, 283, 328, 87, 269, 264, 148, 47, 249, 47, 51, 334, 12, 70])
scrabler.append([287, 210, 66, 10, 384, 160, 279, 308, 111, 318, 298, 185, 200, 36, 88, 131, 91, 304, 126, 337, 17, 213, 101, 290, 394, 139, 289, 333, 185, 136, 196, 139, 102, 276, 164, 115, 43, 334, 299, 261, 319, 242, 67, 81, 17, 133, 80, 200, 95, 358, 207, 74, 198, 86, 117, 289, 3, 96, 74, 230, 249, 29, 104, 327, 276, 368, 354, 253, 268, 245, 201, 377, 367, 58, 274, 70, 224, 189, 222, 0, 335, 391, 144, 28, 223, 93, 138, 192, 42, 146, 150, 383, 385, 104, 75, 121, 232, 4, 58, 329, 308, 311, 329, 32, 78, 229, 282, 67, 153, 213, 221, 184, 199, 332, 183, 291, 46, 304, 305, 336, 365, 346, 7, 211, 384, 294, 34, 331, 11, 55, 32, 2, 144, 213, 304, 7, 86, 174, 60, 344, 178, 220, 25, 379, 31, 31, 375, 15, 134, 221, 355, 114, 264, 298, 326, 180, 202, 137, 20, 252, 18, 347, 63, 352, 305, 143, 316, 368, 75, 182, 35, 161, 146, 225, 341, 322, 167, 22, 42, 262, 308, 310, 23, 108, 55, 361, 181, 91, 234, 99, 365, 19, 90, 165, 212, 293, 393, 264, 384, 277, 351, 52, 7, 398, 342, 18, 297, 294, 142, 183, 142, 290, 300, 208, 363, 150, 30, 90, 62, 194, 4, 96, 279, 163, 114, 337, 98, 48, 135, 119, 36, 381, 297, 365, 165, 220, 6, 340, 43, 16, 145, 76, 274, 218, 9, 358, 317, 176, 175, 354, 128, 135, 132, 316, 52, 364, 228, 95, 1, 85, 322, 98, 184, 11, 31, 99, 346, 328, 315, 174, 361, 395, 375, 25, 177, 77, 85, 180, 10, 53, 186, 396, 251, 231, 121, 382, 105, 14, 188, 228, 333, 317, 388, 118, 376, 26, 266, 246, 195, 77, 250, 51, 346, 108, 382, 387, 357, 95, 281, 324, 261, 151, 391, 41, 255, 235, 269, 62, 85, 166, 303, 53, 119, 294, 365, 234, 369, 255, 159, 110, 296, 121, 110, 313, 346, 294, 301, 282, 168, 40, 307, 44, 324, 151, 341, 398, 148, 292, 312, 3, 119, 201, 288, 255, 93, 375, 283, 249, 75, 400, 58, 76, 141, 178, 386, 28, 43, 57, 260, 156, 62, 10, 292, 381, 295, 2, 174, 326, 67, 177, 275, 361, 293, 208, 385, 154, 63, 88, 256, 76, 160, 308, 131, 43, 172, 199, 143, 54, 314, 102])
scrabler.append([287, 213, 83, 198, 170, 170, 250, 65, 400, 0, 247, 122, 214, 298, 22, 327, 54, 346, 153, 361, 50, 30, 372, 84, 161, 250, 234, 263, 320, 59, 322, 143, 158, 358, 301, 125, 169, 227, 227, 148, 306, 371, 161, 176, 95, 319, 162, 184, 134, 85, 178, 120, 212, 293, 270, 13, 220, 317, 362, 316, 38, 355, 176, 36, 279, 217, 202, 305, 393, 171, 49, 54, 372, 396, 378, 258, 67, 385, 308, 96, 213, 189, 379, 205, 281, 394, 79, 198, 380, 239, 341, 165, 145, 68, 194, 110, 223, 388, 288, 169, 110, 3, 26, 369, 170, 168, 313, 286, 74, 189, 266, 155, 129, 93, 339, 330, 383, 348, 371, 354, 271, 177, 274, 31, 167, 128, 329, 279, 235, 239, 146, 94, 245, 228, 291, 258, 299, 353, 77, 89, 121, 344, 284, 253, 376, 77, 103, 380, 122, 16, 215, 188, 116, 137, 131, 270, 142, 327, 59, 145, 274, 260, 346, 135, 327, 304, 250, 351, 79, 158, 333, 178, 319, 86, 148, 180, 20, 312, 296, 171, 300, 262, 184, 379, 124, 235, 311, 288, 153, 382, 298, 195, 52, 96, 26, 131, 212, 366, 137, 152, 139, 58, 5, 100, 294, 381, 70, 306, 275, 137, 38, 100, 245, 366, 88, 81, 180, 75, 71, 213, 53, 341, 25, 360, 212, 74, 139, 27, 377, 301, 307, 162, 339, 324, 36, 396, 316, 200, 228, 243, 64, 42, 104, 103, 336, 398, 294, 67, 34, 159, 61, 266, 152, 179, 109, 345, 260, 271, 93, 143, 294, 303, 306, 390, 207, 52, 108, 310, 301, 241, 215, 6, 336, 393, 67, 246, 63, 332, 77, 134, 120, 396, 382, 124, 393, 281, 34, 395, 228, 114, 210, 133, 201, 340, 255, 34, 293, 9, 54, 322, 350, 387, 307, 301, 141, 90, 80, 260, 313, 10, 378, 160, 332, 232, 389, 319, 49, 338, 12, 393, 215, 104, 17, 340, 245, 133, 10, 32, 297, 340, 200, 114, 252, 107, 345, 116, 219, 370, 240, 162, 394, 270, 23, 335, 85, 242, 321, 378, 143, 356, 231, 9, 17, 270, 229, 173, 143, 289, 188, 168, 259, 82, 396, 124, 50, 62, 369, 281, 21, 113, 218, 106, 138, 174, 274, 150, 140, 385, 348, 274, 269, 75, 89, 242, 37, 127, 74, 382, 253, 38, 17, 390, 263, 166, 26, 118, 96, 208, 371, 382])
scrabler.append([280, 212, 362, 99, 107, 303, 51, 304, 298, 297, 279, 256, 195, 1, 306, 66, 6, 330, 335, 123, 64, 15, 192, 232, 89, 222, 141, 390, 248, 276, 200, 42, 343, 205, 159, 232, 67, 285, 136, 231, 113, 264, 133, 54, 291, 275, 131, 121, 336, 192, 374, 351, 214, 8, 339, 132, 232, 243, 371, 256, 219, 95, 118, 150, 213, 283, 373, 112, 311, 389, 171, 181, 262, 321, 319, 366, 89, 17, 355, 298, 345, 334, 45, 98, 72, 373, 75, 195, 107, 291, 276, 359, 12, 133, 258, 190, 186, 336, 184, 156, 359, 282, 366, 339, 90, 112, 333, 46, 241, 298, 314, 357, 115, 73, 330, 264, 16, 285, 370, 389, 336, 240, 315, 364, 146, 351, 81, 32, 354, 238, 391, 380, 382, 330, 36, 291, 311, 105, 75, 62, 139, 183, 22, 374, 343, 351, 42, 339, 378, 223, 59, 149, 56, 14, 344, 68, 256, 366, 133, 260, 137, 47, 43, 303, 306, 361, 138, 206, 62, 43, 275, 257, 167, 3, 105, 383, 171, 134, 178, 236, 381, 263, 89, 147, 323, 386, 132, 89, 180, 26, 41, 57, 388, 334, 247, 186, 234, 22, 57, 5, 65, 122, 154, 334, 274, 154, 6, 254, 264, 3, 164, 382, 218, 30, 133, 280, 14, 83, 150, 361, 256, 248, 221, 226, 227, 297, 265, 345, 186, 128, 280, 314, 243, 50, 232, 95, 353, 203, 151, 227, 316, 127, 37, 395, 374, 274, 350, 311, 129, 353, 336, 56, 236, 70, 320, 349, 245, 255, 266, 395, 224, 382, 100, 330, 121, 83, 53, 102, 136, 217, 237, 393, 353, 307, 215, 189, 222, 288, 349, 248, 324, 98, 393, 25, 61, 176, 20, 397, 260, 389, 57, 264, 388, 63, 394, 126, 337, 10, 199, 88, 147, 139, 350, 4, 64, 83, 240, 281, 41, 278, 248, 287, 370, 288, 24, 333, 355, 336, 280, 280, 105, 128, 165, 383, 198, 228, 74, 269, 20, 1, 80, 256, 272, 192, 47, 84, 90, 320, 124, 378, 160, 79, 139, 141, 131, 40, 332, 348, 196, 76, 148, 388, 61, 389, 165, 68, 187, 280, 143, 136, 109, 8, 247, 233, 395, 201, 84, 5, 131, 118, 14, 97, 113, 290, 109, 118, 67, 341, 322, 286, 332, 259, 172, 197, 387, 257, 20, 346, 138, 198, 259, 314, 364, 44, 133, 61, 301, 362, 291, 267])
scrabler.append([376, 55, 188, 53, 90, 21, 120, 173, 264, 383, 46, 336, 98, 105, 89, 106, 56, 341, 261, 302, 176, 385, 330, 397, 184, 138, 93, 358, 274, 35, 61, 284, 14, 385, 349, 203, 159, 140, 208, 349, 194, 214, 234, 7, 390, 364, 138, 342, 391, 195, 118, 313, 186, 170, 36, 91, 18, 68, 187, 384, 183, 87, 352, 142, 206, 36, 10, 306, 218, 149, 221, 176, 25, 59, 317, 26, 360, 164, 137, 129, 226, 156, 15, 148, 2, 41, 347, 217, 29, 269, 188, 270, 398, 183, 323, 138, 265, 308, 80, 273, 209, 116, 30, 335, 163, 245, 204, 75, 38, 81, 307, 203, 271, 110, 57, 206, 327, 361, 209, 92, 182, 103, 324, 33, 342, 28, 148, 94, 246, 286, 389, 7, 307, 238, 316, 243, 339, 246, 175, 209, 244, 251, 52, 40, 301, 268, 189, 28, 292, 61, 55, 327, 67, 108, 220, 297, 377, 24, 329, 292, 246, 167, 175, 305, 320, 86, 321, 25, 295, 35, 208, 141, 195, 109, 113, 64, 9, 388, 27, 181, 171, 45, 281, 160, 274, 306, 10, 36, 391, 382, 353, 157, 51, 135, 15, 293, 263, 336, 321, 172, 133, 168, 386, 160, 132, 314, 157, 5, 205, 305, 305, 229, 188, 226, 203, 21, 141, 325, 241, 23, 154, 279, 267, 126, 310, 236, 157, 60, 253, 39, 228, 173, 198, 281, 179, 91, 43, 217, 157, 263, 56, 92, 207, 12, 202, 218, 315, 102, 58, 55, 150, 39, 278, 195, 50, 189, 31, 116, 316, 276, 376, 13, 293, 251, 165, 322, 279, 64, 350, 204, 193, 199, 272, 131, 177, 326, 168, 289, 323, 41, 58, 348, 294, 308, 365, 329, 315, 0, 289, 381, 387, 245, 0, 274, 99, 162, 103, 72, 44, 284, 189, 126, 209, 279, 254, 159, 229, 277, 202, 239, 331, 60, 224, 92, 235, 45, 195, 58, 137, 246, 292, 186, 361, 211, 138, 230, 387, 154, 69, 294, 46, 15, 368, 385, 107, 88, 7, 82, 364, 83, 286, 242, 398, 17, 118, 295, 338, 262, 296, 181, 72, 155, 69, 301, 74, 42, 308, 178, 262, 333, 348, 363, 172, 384, 342, 340, 336, 325, 108, 344, 330, 392, 162, 212, 264, 315, 196, 172, 14, 78, 184, 147, 247, 212, 390, 41, 128, 306, 323, 234, 284, 200, 308, 215, 167, 342, 343, 141, 14, 339])
scrabler.append([107, 355, 201, 359, 148, 370, 281, 126, 44, 341, 286, 51, 283, 29, 273, 250, 100, 104, 90, 259, 212, 302, 9, 322, 358, 16, 282, 14, 58, 193, 306, 107, 272, 177, 118, 140, 266, 321, 310, 288, 381, 292, 52, 348, 391, 255, 272, 50, 67, 374, 184, 299, 175, 89, 342, 249, 219, 185, 291, 80, 384, 244, 106, 137, 287, 66, 42, 167, 371, 14, 43, 316, 358, 24, 18, 352, 203, 13, 3, 204, 166, 170, 329, 30, 267, 44, 325, 167, 120, 264, 57, 81, 280, 221, 314, 382, 2, 122, 147, 311, 277, 63, 168, 186, 257, 101, 179, 264, 7, 93, 181, 308, 25, 193, 363, 399, 120, 120, 358, 82, 73, 17, 182, 21, 399, 212, 270, 96, 356, 130, 34, 87, 391, 137, 102, 293, 49, 115, 124, 162, 316, 320, 161, 178, 127, 229, 301, 276, 220, 284, 126, 338, 385, 11, 161, 189, 366, 244, 145, 336, 389, 306, 261, 87, 276, 271, 80, 2, 237, 342, 359, 200, 276, 222, 115, 281, 5, 174, 88, 28, 101, 385, 310, 363, 69, 389, 37, 312, 126, 229, 244, 153, 48, 290, 129, 378, 0, 40, 106, 223, 98, 173, 239, 26, 16, 257, 107, 212, 238, 212, 313, 131, 103, 4, 144, 129, 263, 385, 217, 301, 155, 78, 219, 36, 150, 71, 30, 236, 160, 220, 305, 115, 367, 8, 348, 173, 289, 63, 83, 69, 158, 273, 216, 257, 386, 150, 382, 269, 314, 280, 157, 85, 353, 97, 335, 368, 263, 159, 223, 88, 319, 361, 90, 64, 307, 110, 204, 193, 97, 187, 162, 335, 190, 194, 45, 14, 164, 153, 146, 11, 155, 199, 338, 361, 130, 376, 246, 74, 119, 288, 189, 94, 194, 190, 55, 389, 50, 60, 272, 162, 43, 89, 158, 29, 1, 268, 38, 2, 362, 165, 132, 164, 171, 212, 182, 3, 68, 103, 217, 182, 67, 130, 326, 90, 110, 214, 13, 116, 278, 238, 307, 312, 366, 96, 307, 340, 352, 191, 81, 70, 141, 319, 9, 17, 270, 374, 281, 309, 291, 256, 159, 49, 282, 306, 327, 358, 167, 125, 138, 134, 221, 187, 74, 215, 146, 168, 342, 228, 51, 400, 198, 325, 90, 347, 323, 312, 99, 292, 13, 67, 127, 168, 322, 236, 27, 333, 272, 42, 169, 158, 258, 328, 103, 50, 289, 142, 80, 299, 255, 102])
scrabler.append([391, 323, 77, 154, 174, 161, 376, 22, 324, 386, 394, 44, 295, 349, 320, 258, 19, 91, 233, 269, 315, 74, 166, 230, 68, 58, 182, 90, 77, 350, 388, 224, 84, 393, 136, 238, 331, 383, 120, 219, 291, 298, 155, 131, 335, 80, 325, 381, 102, 84, 6, 132, 71, 129, 308, 335, 371, 27, 86, 320, 44, 231, 73, 247, 33, 60, 144, 283, 77, 113, 341, 369, 391, 236, 209, 290, 220, 376, 236, 24, 208, 205, 46, 32, 71, 13, 80, 371, 236, 28, 90, 192, 400, 28, 157, 163, 40, 84, 287, 265, 115, 310, 109, 250, 261, 346, 210, 38, 113, 289, 115, 359, 231, 43, 269, 149, 271, 217, 351, 326, 23, 143, 325, 177, 165, 305, 238, 196, 352, 186, 87, 77, 119, 106, 365, 279, 268, 175, 346, 314, 131, 133, 60, 88, 174, 133, 225, 174, 130, 17, 222, 40, 25, 45, 201, 352, 314, 229, 138, 153, 362, 30, 4, 300, 6, 386, 325, 322, 20, 113, 302, 370, 384, 201, 373, 71, 139, 192, 52, 303, 116, 257, 151, 85, 85, 184, 206, 182, 7, 181, 48, 385, 11, 8, 317, 290, 368, 311, 340, 58, 370, 359, 108, 38, 300, 368, 95, 366, 393, 356, 288, 186, 370, 84, 102, 240, 43, 144, 376, 274, 326, 266, 365, 123, 22, 299, 117, 188, 88, 370, 20, 180, 33, 268, 388, 216, 189, 361, 130, 380, 194, 72, 237, 322, 116, 125, 194, 89, 47, 334, 275, 283, 102, 64, 211, 390, 386, 372, 389, 264, 225, 371, 182, 160, 376, 141, 53, 156, 276, 85, 47, 209, 260, 8, 375, 199, 85, 42, 379, 182, 263, 195, 283, 12, 341, 155, 316, 260, 212, 119, 30, 380, 322, 170, 341, 298, 12, 216, 394, 42, 358, 158, 252, 363, 321, 109, 40, 348, 314, 351, 55, 143, 95, 373, 93, 101, 64, 252, 67, 309, 298, 295, 301, 191, 398, 203, 96, 359, 274, 400, 69, 186, 6, 224, 120, 292, 45, 5, 99, 352, 305, 208, 349, 347, 162, 71, 237, 36, 226, 267, 18, 11, 181, 12, 80, 208, 290, 154, 28, 358, 241, 14, 16, 210, 311, 120, 290, 33, 287, 43, 224, 97, 235, 319, 13, 115, 280, 280, 156, 23, 378, 121, 33, 310, 58, 181, 165, 293, 385, 102, 37, 15, 73, 280, 7, 352, 213, 358, 13, 76])
scrabler.append([167, 326, 80, 370, 165, 110, 337, 385, 187, 103, 162, 357, 267, 126, 114, 70, 297, 398, 162, 175, 170, 123, 337, 9, 393, 24, 265, 276, 90, 300, 191, 10, 131, 113, 49, 30, 372, 108, 143, 127, 86, 302, 347, 236, 155, 0, 286, 32, 340, 50, 20, 131, 254, 147, 259, 213, 228, 290, 298, 374, 311, 235, 51, 38, 164, 66, 293, 235, 62, 107, 174, 383, 312, 161, 66, 102, 345, 382, 279, 368, 16, 196, 151, 240, 276, 263, 268, 208, 174, 35, 179, 60, 307, 260, 396, 158, 63, 82, 57, 139, 387, 211, 223, 152, 187, 340, 18, 345, 2, 363, 233, 84, 68, 289, 295, 120, 98, 282, 3, 5, 157, 91, 339, 59, 57, 329, 142, 165, 345, 236, 45, 295, 233, 141, 10, 356, 351, 250, 357, 168, 70, 266, 33, 323, 358, 39, 93, 130, 365, 312, 192, 85, 254, 17, 259, 247, 323, 123, 290, 105, 186, 211, 271, 175, 31, 32, 245, 95, 284, 81, 375, 225, 73, 105, 112, 394, 60, 45, 200, 295, 49, 198, 144, 222, 71, 136, 167, 38, 379, 251, 242, 304, 336, 15, 255, 234, 24, 97, 270, 334, 265, 362, 266, 370, 87, 68, 260, 148, 298, 389, 89, 280, 99, 32, 298, 146, 166, 5, 147, 193, 387, 263, 65, 132, 256, 224, 174, 394, 103, 131, 229, 378, 49, 36, 318, 51, 291, 326, 244, 313, 340, 288, 201, 18, 29, 392, 187, 322, 34, 214, 6, 110, 276, 58, 140, 303, 53, 205, 256, 289, 286, 144, 321, 243, 177, 15, 64, 185, 31, 76, 343, 292, 90, 149, 298, 330, 75, 357, 173, 383, 160, 149, 123, 195, 214, 311, 224, 381, 317, 208, 155, 250, 343, 79, 94, 328, 290, 313, 59, 380, 241, 59, 11, 46, 257, 194, 234, 337, 247, 85, 42, 25, 80, 153, 119, 199, 205, 345, 238, 283, 268, 153, 196, 236, 388, 360, 126, 225, 187, 265, 226, 275, 14, 286, 270, 390, 42, 286, 325, 64, 23, 158, 104, 298, 344, 241, 307, 323, 332, 363, 134, 82, 16, 355, 351, 72, 204, 175, 127, 226, 55, 129, 338, 400, 332, 287, 27, 64, 281, 380, 247, 66, 259, 258, 166, 52, 44, 208, 143, 339, 356, 288, 169, 261, 374, 55, 202, 281, 183, 93, 270, 319, 87, 247, 364, 147, 276, 383, 61, 118])
scrabler.append([101, 213, 242, 53, 241, 259, 39, 167, 313, 390, 108, 29, 267, 116, 372, 121, 205, 399, 34, 399, 309, 246, 236, 210, 149, 323, 393, 324, 320, 284, 54, 188, 374, 181, 190, 209, 336, 114, 200, 285, 110, 78, 67, 254, 257, 196, 172, 105, 13, 162, 273, 344, 162, 0, 296, 239, 171, 185, 124, 99, 372, 250, 122, 164, 142, 240, 338, 351, 362, 329, 329, 10, 373, 53, 253, 135, 265, 48, 340, 288, 149, 388, 154, 230, 71, 271, 364, 336, 210, 350, 270, 176, 180, 134, 297, 36, 306, 250, 85, 47, 184, 325, 140, 221, 133, 1, 259, 271, 7, 116, 116, 77, 391, 359, 355, 350, 274, 183, 90, 281, 219, 94, 296, 88, 340, 280, 383, 328, 50, 74, 102, 260, 353, 32, 4, 279, 41, 399, 104, 325, 273, 290, 193, 109, 42, 268, 62, 33, 279, 139, 129, 262, 39, 241, 5, 285, 68, 95, 333, 203, 70, 383, 103, 193, 231, 140, 89, 355, 300, 377, 155, 325, 187, 125, 396, 159, 42, 298, 3, 265, 394, 218, 16, 169, 256, 185, 177, 153, 393, 75, 226, 264, 159, 367, 340, 190, 370, 19, 108, 161, 123, 187, 150, 116, 342, 374, 393, 342, 102, 225, 181, 158, 99, 163, 106, 73, 3, 296, 334, 254, 358, 339, 310, 105, 364, 388, 264, 163, 213, 364, 332, 376, 87, 265, 31, 273, 305, 282, 115, 367, 285, 148, 307, 61, 100, 314, 182, 49, 147, 238, 297, 288, 154, 56, 280, 275, 137, 113, 104, 367, 28, 19, 134, 69, 395, 306, 318, 101, 243, 281, 53, 324, 17, 124, 236, 50, 201, 68, 216, 346, 21, 213, 264, 182, 109, 271, 113, 309, 360, 192, 220, 120, 226, 342, 140, 235, 188, 198, 251, 312, 332, 356, 118, 111, 140, 110, 37, 366, 100, 282, 313, 352, 160, 391, 132, 114, 219, 19, 38, 394, 256, 10, 102, 292, 4, 367, 261, 221, 72, 57, 346, 230, 57, 288, 116, 147, 254, 154, 71, 11, 315, 226, 186, 57, 294, 206, 183, 242, 165, 4, 129, 287, 288, 318, 85, 214, 138, 210, 208, 112, 249, 329, 233, 9, 6, 369, 99, 32, 364, 274, 107, 25, 29, 107, 31, 124, 112, 397, 100, 140, 221, 119, 379, 352, 161, 343, 161, 314, 322, 87, 220, 142, 272, 49, 72, 44, 338, 225, 108, 350])
scrabler.append([54, 166, 47, 11, 307, 284, 77, 262, 279, 349, 217, 190, 102, 230, 120, 228, 61, 194, 4, 276, 366, 347, 292, 164, 148, 56, 253, 370, 264, 238, 300, 352, 317, 274, 9, 137, 302, 103, 169, 337, 201, 256, 200, 228, 345, 76, 5, 349, 219, 155, 250, 399, 4, 91, 222, 65, 67, 86, 324, 254, 284, 398, 103, 90, 94, 302, 266, 191, 28, 140, 51, 330, 278, 109, 319, 300, 381, 51, 119, 329, 253, 207, 365, 318, 389, 348, 94, 119, 387, 251, 170, 324, 108, 105, 174, 193, 178, 109, 125, 358, 84, 46, 137, 384, 140, 386, 386, 125, 182, 25, 20, 362, 299, 260, 92, 198, 392, 88, 185, 81, 26, 191, 5, 308, 16, 37, 184, 385, 154, 361, 29, 21, 256, 21, 329, 213, 19, 263, 392, 8, 37, 55, 36, 241, 88, 201, 354, 155, 329, 97, 383, 244, 265, 82, 145, 351, 125, 311, 59, 45, 39, 247, 123, 20, 358, 65, 285, 38, 324, 198, 260, 262, 56, 265, 197, 321, 41, 71, 52, 308, 155, 75, 147, 232, 169, 129, 371, 270, 137, 20, 11, 217, 74, 52, 131, 5, 200, 31, 382, 97, 308, 336, 400, 53, 109, 325, 173, 222, 76, 228, 191, 344, 315, 18, 285, 324, 210, 397, 301, 108, 108, 314, 324, 118, 55, 173, 363, 251, 361, 236, 19, 108, 221, 195, 377, 122, 184, 286, 19, 184, 70, 324, 80, 391, 302, 263, 172, 329, 221, 51, 150, 240, 168, 231, 240, 249, 316, 374, 335, 249, 210, 3, 313, 155, 158, 360, 315, 179, 157, 257, 267, 89, 282, 290, 89, 393, 83, 228, 36, 279, 53, 161, 271, 335, 358, 315, 278, 392, 129, 190, 183, 32, 33, 151, 269, 162, 12, 284, 92, 309, 389, 243, 191, 391, 287, 166, 253, 62, 219, 252, 359, 339, 160, 90, 321, 151, 79, 197, 287, 27, 97, 8, 58, 274, 78, 33, 355, 281, 97, 84, 64, 220, 311, 282, 50, 307, 156, 319, 68, 43, 86, 300, 218, 78, 130, 73, 147, 128, 282, 158, 81, 176, 380, 356, 94, 205, 294, 349, 177, 365, 271, 4, 269, 62, 400, 128, 96, 93, 150, 267, 302, 242, 321, 203, 179, 372, 217, 4, 226, 96, 382, 377, 95, 3, 23, 335, 264, 149, 375, 195, 290, 238, 123, 133, 395, 184, 272, 91, 171, 394])
scrabler.append([64, 143, 91, 276, 250, 279, 121, 24, 365, 116, 391, 272, 49, 273, 351, 308, 163, 262, 236, 183, 0, 58, 127, 309, 387, 198, 207, 230, 221, 320, 7, 375, 127, 397, 253, 318, 396, 298, 294, 39, 196, 43, 323, 229, 145, 73, 38, 372, 105, 330, 170, 17, 325, 212, 70, 167, 124, 124, 237, 66, 125, 65, 231, 321, 56, 352, 18, 393, 388, 343, 78, 344, 103, 278, 108, 166, 284, 250, 147, 395, 334, 123, 327, 274, 135, 378, 356, 183, 44, 52, 125, 18, 91, 199, 87, 388, 314, 391, 303, 326, 172, 175, 40, 231, 114, 344, 213, 399, 225, 204, 186, 106, 61, 396, 362, 112, 347, 135, 271, 92, 368, 146, 224, 46, 25, 71, 14, 263, 232, 118, 38, 259, 318, 307, 162, 9, 330, 70, 172, 28, 246, 400, 322, 313, 208, 297, 147, 103, 210, 73, 144, 110, 112, 309, 364, 327, 121, 398, 254, 200, 195, 65, 366, 86, 330, 398, 81, 180, 136, 98, 47, 153, 272, 328, 14, 349, 65, 278, 56, 208, 334, 318, 64, 10, 35, 29, 171, 98, 75, 184, 38, 171, 298, 167, 368, 280, 277, 378, 197, 288, 351, 329, 26, 327, 258, 161, 280, 38, 353, 48, 319, 52, 113, 180, 298, 91, 151, 30, 235, 94, 349, 27, 71, 227, 93, 105, 260, 68, 58, 297, 46, 113, 147, 164, 80, 33, 266, 331, 50, 95, 123, 367, 73, 251, 143, 81, 213, 128, 237, 197, 376, 12, 228, 205, 389, 143, 255, 231, 390, 97, 185, 157, 264, 338, 241, 301, 68, 167, 312, 344, 179, 28, 299, 386, 250, 205, 39, 39, 313, 277, 350, 131, 233, 27, 3, 24, 206, 10, 24, 297, 61, 59, 283, 280, 375, 156, 362, 34, 7, 126, 215, 203, 300, 391, 2, 247, 103, 300, 7, 184, 344, 19, 147, 321, 167, 355, 42, 44, 132, 52, 323, 124, 232, 391, 254, 184, 213, 390, 186, 223, 351, 123, 392, 354, 283, 239, 116, 192, 277, 294, 302, 106, 320, 116, 217, 79, 257, 362, 329, 262, 164, 246, 200, 146, 0, 18, 149, 2, 234, 180, 266, 388, 363, 313, 168, 332, 334, 185, 137, 238, 14, 343, 152, 319, 285, 0, 351, 323, 159, 322, 61, 293, 95, 278, 117, 366, 229, 329, 7, 90, 367, 149, 159, 138, 277, 389, 400, 382, 377, 284])
scrabler.append([24, 102, 60, 133, 87, 39, 314, 399, 157, 200, 317, 207, 59, 93, 340, 34, 397, 32, 252, 289, 359, 68, 103, 287, 176, 316, 373, 270, 224, 270, 120, 266, 244, 210, 116, 66, 38, 310, 166, 182, 141, 176, 36, 108, 322, 94, 133, 139, 52, 104, 370, 50, 199, 147, 201, 194, 100, 376, 138, 197, 171, 101, 174, 76, 356, 284, 210, 286, 310, 50, 389, 273, 86, 254, 208, 326, 354, 223, 158, 355, 358, 293, 245, 136, 103, 353, 267, 36, 371, 295, 295, 174, 305, 361, 19, 26, 290, 50, 182, 102, 72, 350, 220, 378, 79, 45, 191, 241, 69, 2, 96, 337, 105, 69, 223, 148, 281, 156, 213, 367, 247, 263, 281, 11, 333, 374, 328, 314, 316, 346, 44, 83, 364, 128, 260, 44, 369, 262, 90, 374, 297, 162, 259, 82, 167, 276, 334, 63, 195, 123, 12, 47, 108, 105, 139, 221, 220, 63, 33, 161, 30, 79, 216, 99, 300, 230, 323, 197, 122, 79, 62, 173, 350, 323, 258, 184, 218, 288, 197, 318, 370, 267, 331, 370, 371, 199, 280, 290, 214, 376, 88, 82, 98, 251, 265, 113, 240, 135, 123, 326, 208, 101, 84, 161, 239, 25, 178, 131, 135, 318, 71, 355, 175, 59, 70, 239, 309, 380, 273, 350, 301, 141, 60, 215, 313, 139, 27, 274, 84, 273, 112, 214, 349, 205, 319, 289, 310, 379, 289, 72, 89, 41, 153, 378, 235, 76, 89, 2, 142, 197, 245, 202, 207, 259, 239, 239, 325, 156, 51, 122, 129, 203, 289, 267, 151, 367, 370, 29, 261, 392, 190, 31, 379, 307, 136, 294, 277, 275, 231, 349, 39, 291, 33, 265, 109, 350, 165, 190, 355, 153, 211, 211, 140, 40, 247, 180, 383, 370, 356, 371, 5, 272, 56, 70, 337, 0, 283, 317, 250, 179, 280, 295, 317, 236, 121, 161, 275, 222, 352, 270, 278, 326, 333, 151, 323, 76, 272, 127, 144, 337, 285, 81, 343, 274, 183, 339, 218, 211, 249, 80, 109, 285, 327, 123, 75, 284, 91, 299, 230, 77, 173, 376, 355, 170, 246, 201, 116, 354, 156, 251, 365, 70, 310, 9, 95, 390, 153, 307, 370, 322, 36, 340, 360, 69, 260, 236, 249, 41, 63, 339, 7, 390, 276, 63, 272, 331, 144, 193, 268, 279, 47, 362, 347, 290, 378, 204, 354, 287, 242, 354])
scrabler.append([83, 119, 232, 23, 91, 245, 283, 64, 79, 266, 285, 202, 222, 349, 0, 11, 160, 29, 29, 217, 316, 20, 46, 102, 372, 288, 334, 199, 136, 334, 322, 38, 41, 128, 105, 36, 92, 67, 69, 372, 270, 339, 379, 301, 115, 168, 248, 293, 81, 268, 264, 260, 290, 308, 275, 182, 34, 127, 96, 254, 247, 273, 3, 129, 329, 344, 37, 41, 251, 364, 69, 81, 342, 59, 388, 38, 81, 397, 193, 18, 207, 134, 68, 361, 282, 70, 160, 227, 98, 279, 270, 359, 363, 46, 171, 282, 301, 121, 368, 275, 41, 182, 101, 214, 170, 72, 48, 150, 84, 158, 40, 328, 384, 85, 76, 326, 143, 286, 44, 263, 16, 3, 127, 279, 212, 75, 385, 344, 91, 290, 365, 17, 285, 324, 284, 143, 399, 379, 65, 396, 107, 292, 204, 267, 105, 150, 73, 107, 259, 375, 299, 142, 352, 230, 196, 144, 137, 116, 42, 129, 74, 87, 233, 238, 128, 108, 44, 358, 30, 255, 154, 371, 59, 16, 219, 166, 344, 278, 120, 356, 306, 109, 208, 28, 104, 235, 281, 350, 368, 148, 218, 87, 209, 149, 179, 92, 63, 177, 326, 77, 341, 345, 124, 384, 249, 45, 54, 25, 200, 207, 99, 102, 244, 346, 393, 42, 235, 288, 204, 188, 300, 326, 218, 304, 167, 150, 363, 226, 193, 140, 350, 153, 45, 56, 387, 122, 223, 152, 296, 308, 196, 30, 367, 84, 72, 5, 192, 353, 35, 85, 330, 107, 381, 157, 219, 63, 92, 336, 84, 192, 136, 88, 0, 313, 28, 68, 20, 134, 193, 293, 227, 110, 143, 259, 82, 205, 308, 323, 340, 246, 211, 78, 21, 323, 196, 242, 207, 398, 234, 172, 286, 201, 389, 167, 62, 129, 395, 245, 338, 3, 101, 2, 270, 246, 227, 270, 252, 213, 260, 276, 391, 374, 10, 47, 391, 141, 2, 366, 397, 49, 39, 342, 210, 229, 391, 163, 41, 229, 59, 178, 77, 1, 114, 153, 132, 274, 306, 193, 189, 324, 129, 85, 180, 249, 283, 31, 96, 42, 378, 255, 342, 278, 127, 304, 185, 320, 8, 305, 137, 317, 93, 153, 227, 40, 315, 365, 128, 15, 82, 358, 137, 109, 386, 260, 332, 96, 7, 14, 20, 3, 322, 47, 281, 328, 394, 17, 34, 111, 66, 349, 142, 226, 203, 160, 393, 90, 119, 353, 338, 151])
scrabler.append([310, 393, 86, 178, 373, 141, 17, 271, 111, 270, 85, 277, 341, 285, 266, 95, 398, 63, 121, 397, 186, 362, 141, 319, 248, 307, 385, 142, 369, 8, 338, 366, 357, 13, 289, 268, 228, 304, 178, 197, 71, 161, 280, 13, 319, 353, 129, 105, 5, 334, 378, 137, 21, 323, 55, 38, 271, 218, 321, 14, 370, 296, 111, 56, 230, 394, 362, 161, 88, 376, 302, 388, 138, 132, 43, 62, 238, 379, 230, 183, 9, 357, 182, 256, 228, 215, 391, 31, 37, 260, 278, 174, 129, 383, 272, 349, 382, 343, 61, 15, 335, 114, 294, 242, 53, 171, 273, 314, 163, 64, 87, 303, 284, 42, 378, 303, 10, 78, 67, 381, 359, 244, 214, 354, 343, 345, 394, 213, 62, 181, 102, 362, 123, 140, 145, 207, 357, 310, 392, 161, 259, 382, 189, 186, 275, 210, 133, 302, 110, 65, 382, 105, 219, 74, 279, 263, 207, 14, 126, 353, 190, 295, 68, 241, 155, 121, 276, 353, 63, 207, 142, 12, 255, 6, 369, 201, 394, 134, 85, 109, 395, 149, 392, 381, 380, 88, 211, 107, 267, 257, 321, 217, 51, 55, 116, 33, 310, 189, 313, 106, 103, 104, 125, 322, 284, 39, 275, 370, 223, 338, 93, 269, 321, 281, 67, 222, 8, 312, 335, 301, 252, 261, 357, 148, 284, 283, 153, 179, 163, 348, 41, 258, 129, 231, 199, 139, 209, 308, 139, 370, 319, 296, 76, 236, 317, 344, 169, 347, 176, 350, 186, 274, 221, 380, 269, 234, 120, 194, 309, 306, 381, 385, 78, 266, 250, 143, 380, 0, 304, 233, 394, 87, 158, 270, 157, 260, 82, 303, 307, 108, 70, 386, 273, 256, 15, 127, 306, 207, 170, 90, 157, 193, 259, 234, 168, 82, 56, 69, 327, 341, 285, 59, 286, 214, 334, 136, 291, 225, 122, 339, 133, 245, 126, 310, 131, 92, 325, 173, 353, 384, 104, 187, 272, 113, 52, 271, 370, 172, 28, 178, 44, 4, 6, 63, 172, 205, 279, 205, 242, 116, 357, 179, 368, 191, 392, 123, 21, 333, 253, 381, 214, 163, 147, 61, 99, 239, 298, 155, 17, 355, 233, 351, 346, 211, 26, 393, 103, 164, 333, 19, 244, 309, 276, 257, 199, 151, 67, 120, 126, 185, 22, 33, 360, 77, 281, 237, 318, 206, 106, 304, 199, 378, 389, 42, 64, 9, 175, 149, 340, 11])
scrabler.append([291, 164, 382, 127, 349, 382, 349, 29, 237, 244, 337, 178, 53, 86, 190, 16, 397, 289, 292, 360, 375, 7, 201, 74, 359, 213, 89, 306, 48, 229, 164, 193, 72, 205, 290, 144, 290, 263, 1, 44, 13, 79, 237, 86, 129, 210, 80, 280, 33, 375, 319, 110, 48, 60, 258, 294, 251, 177, 20, 228, 338, 163, 113, 254, 277, 327, 262, 374, 314, 288, 333, 66, 60, 56, 160, 74, 133, 42, 114, 356, 267, 158, 269, 88, 134, 136, 131, 366, 349, 49, 5, 283, 6, 106, 78, 188, 370, 40, 305, 163, 121, 64, 119, 288, 359, 352, 111, 21, 245, 313, 253, 383, 217, 17, 223, 324, 380, 246, 0, 147, 342, 16, 354, 91, 57, 52, 120, 77, 29, 335, 327, 295, 351, 224, 382, 371, 227, 252, 392, 68, 362, 392, 82, 232, 69, 385, 291, 162, 222, 209, 138, 28, 170, 4, 262, 68, 38, 328, 294, 345, 78, 206, 339, 182, 314, 396, 117, 232, 394, 326, 267, 235, 60, 374, 98, 203, 65, 141, 67, 174, 299, 193, 67, 160, 168, 399, 142, 109, 148, 329, 149, 230, 212, 288, 335, 398, 214, 204, 30, 104, 64, 36, 292, 352, 348, 194, 155, 276, 221, 14, 354, 203, 13, 391, 158, 246, 160, 99, 184, 54, 241, 188, 12, 345, 39, 197, 220, 204, 67, 346, 165, 193, 170, 345, 380, 111, 326, 110, 164, 276, 218, 177, 219, 224, 73, 343, 296, 73, 98, 260, 65, 388, 396, 92, 230, 262, 378, 252, 216, 273, 337, 2, 177, 70, 320, 122, 292, 385, 223, 309, 133, 117, 136, 45, 276, 125, 186, 385, 237, 56, 157, 76, 79, 131, 122, 283, 295, 334, 207, 124, 378, 177, 289, 168, 127, 1, 340, 195, 267, 43, 330, 379, 189, 149, 282, 328, 38, 4, 172, 125, 106, 277, 107, 289, 304, 33, 84, 348, 330, 96, 175, 161, 90, 128, 178, 1, 137, 47, 83, 223, 242, 61, 35, 28, 343, 325, 235, 170, 249, 356, 95, 299, 274, 221, 296, 225, 45, 17, 367, 86, 234, 182, 28, 343, 94, 19, 198, 50, 387, 392, 289, 56, 288, 350, 8, 196, 172, 25, 355, 187, 321, 140, 136, 157, 67, 388, 354, 244, 171, 299, 184, 348, 354, 15, 220, 291, 261, 289, 321, 284, 94, 373, 374, 340, 368, 197, 83, 77, 88, 57])
scrabler.append([11, 371, 224, 254, 263, 315, 354, 223, 68, 101, 177, 154, 153, 65, 73, 28, 287, 8, 206, 244, 38, 288, 31, 156, 12, 98, 297, 58, 152, 340, 117, 272, 106, 94, 327, 85, 116, 236, 353, 80, 337, 277, 228, 257, 145, 117, 181, 200, 353, 341, 145, 11, 94, 317, 11, 103, 198, 13, 104, 26, 395, 330, 126, 216, 10, 355, 18, 159, 165, 236, 201, 310, 253, 119, 389, 84, 213, 161, 251, 88, 36, 350, 389, 214, 197, 39, 282, 125, 34, 189, 276, 76, 147, 57, 318, 341, 40, 355, 124, 318, 392, 91, 55, 76, 328, 244, 383, 374, 288, 364, 367, 81, 140, 400, 91, 350, 289, 212, 338, 330, 312, 127, 252, 255, 285, 189, 22, 316, 270, 329, 192, 166, 234, 145, 176, 377, 134, 26, 339, 348, 17, 386, 110, 271, 306, 394, 388, 264, 137, 44, 11, 357, 386, 187, 222, 197, 325, 219, 298, 66, 253, 254, 283, 43, 306, 87, 81, 361, 284, 7, 321, 12, 87, 163, 97, 233, 194, 36, 381, 95, 143, 390, 54, 90, 215, 154, 251, 181, 384, 323, 224, 290, 271, 379, 133, 25, 291, 256, 141, 272, 189, 77, 104, 3, 314, 304, 105, 95, 38, 352, 156, 225, 294, 142, 254, 330, 390, 280, 6, 334, 42, 342, 240, 293, 122, 267, 224, 273, 79, 99, 112, 170, 321, 374, 311, 218, 386, 130, 150, 193, 399, 3, 259, 127, 2, 385, 220, 328, 216, 389, 56, 226, 271, 351, 168, 37, 90, 278, 319, 289, 272, 230, 206, 275, 296, 125, 184, 387, 49, 329, 235, 186, 188, 173, 381, 293, 222, 388, 395, 358, 148, 310, 354, 168, 248, 266, 176, 33, 45, 121, 140, 265, 378, 86, 32, 355, 236, 347, 149, 265, 322, 280, 318, 38, 173, 387, 189, 41, 82, 236, 18, 177, 264, 343, 289, 10, 8, 372, 172, 308, 107, 28, 341, 341, 295, 120, 256, 270, 91, 13, 60, 143, 231, 74, 290, 391, 133, 122, 328, 112, 324, 398, 97, 84, 281, 198, 19, 166, 106, 200, 2, 385, 116, 345, 239, 233, 290, 336, 302, 29, 199, 318, 303, 63, 140, 148, 311, 0, 35, 15, 358, 256, 173, 266, 362, 124, 17, 111, 134, 329, 181, 373, 279, 182, 30, 57, 96, 86, 73, 92, 195, 302, 223, 98, 129, 311, 336, 94, 331, 13])
scrabler.append([274, 19, 48, 292, 400, 293, 150, 227, 226, 194, 73, 49, 276, 136, 361, 154, 50, 316, 318, 173, 44, 15, 150, 119, 53, 273, 98, 7, 305, 163, 225, 389, 371, 127, 387, 98, 227, 130, 261, 111, 110, 43, 225, 145, 61, 51, 121, 69, 127, 317, 283, 65, 334, 222, 128, 9, 47, 324, 347, 134, 320, 74, 376, 374, 361, 385, 349, 268, 383, 71, 384, 223, 163, 311, 146, 120, 60, 147, 228, 26, 287, 89, 248, 366, 191, 174, 194, 319, 251, 252, 338, 20, 223, 73, 97, 299, 152, 375, 83, 32, 313, 390, 252, 381, 6, 144, 114, 249, 7, 25, 149, 75, 379, 312, 323, 198, 79, 213, 82, 378, 344, 279, 329, 147, 70, 12, 339, 382, 87, 8, 98, 19, 5, 323, 269, 49, 77, 238, 278, 35, 283, 170, 134, 279, 163, 173, 231, 159, 334, 210, 323, 158, 39, 317, 17, 215, 35, 295, 321, 86, 74, 273, 296, 35, 370, 34, 252, 198, 372, 193, 21, 367, 355, 162, 34, 303, 87, 145, 330, 51, 241, 105, 197, 57, 391, 381, 164, 76, 35, 3, 369, 303, 332, 160, 89, 138, 18, 342, 268, 246, 264, 18, 181, 365, 195, 190, 327, 207, 51, 109, 10, 381, 125, 274, 91, 70, 251, 234, 107, 392, 320, 326, 111, 117, 295, 231, 104, 381, 310, 397, 373, 150, 78, 23, 390, 110, 167, 379, 240, 52, 59, 349, 60, 132, 335, 77, 210, 397, 162, 392, 10, 44, 364, 62, 163, 47, 9, 301, 103, 146, 345, 213, 314, 349, 19, 388, 23, 378, 92, 13, 67, 54, 82, 4, 326, 392, 395, 28, 33, 349, 161, 310, 147, 59, 143, 155, 153, 155, 200, 34, 148, 259, 40, 113, 9, 216, 43, 79, 201, 398, 210, 399, 49, 154, 199, 397, 173, 347, 393, 157, 322, 128, 272, 157, 90, 125, 232, 132, 178, 30, 144, 103, 198, 235, 279, 307, 184, 235, 280, 267, 101, 278, 112, 120, 216, 158, 233, 120, 262, 257, 10, 357, 57, 369, 35, 239, 202, 162, 115, 0, 245, 128, 42, 121, 240, 149, 15, 295, 333, 60, 379, 48, 71, 268, 29, 292, 286, 17, 23, 94, 152, 379, 329, 140, 181, 9, 201, 64, 353, 227, 325, 318, 346, 36, 143, 254, 106, 7, 99, 189, 41, 64, 27, 361, 127, 327, 391, 47, 331, 303])
scrabler.append([87, 87, 376, 389, 260, 40, 270, 15, 364, 5, 215, 20, 233, 366, 24, 226, 93, 240, 378, 97, 148, 45, 259, 308, 218, 109, 264, 208, 194, 83, 209, 124, 318, 219, 384, 333, 214, 344, 365, 248, 89, 266, 306, 25, 46, 38, 102, 78, 363, 158, 377, 222, 104, 174, 102, 297, 3, 273, 184, 318, 306, 11, 176, 187, 267, 105, 242, 51, 152, 284, 291, 349, 186, 216, 366, 185, 239, 321, 117, 84, 14, 261, 90, 214, 126, 2, 20, 179, 127, 322, 218, 213, 71, 4, 113, 173, 27, 332, 349, 88, 251, 329, 171, 58, 150, 250, 115, 162, 5, 64, 74, 184, 97, 131, 77, 8, 385, 347, 363, 165, 289, 73, 12, 323, 264, 328, 58, 120, 395, 343, 83, 192, 388, 257, 68, 354, 299, 325, 252, 201, 389, 262, 280, 162, 325, 285, 232, 131, 276, 287, 343, 283, 65, 18, 146, 394, 265, 282, 114, 48, 125, 357, 27, 229, 354, 365, 107, 370, 17, 220, 99, 51, 41, 318, 222, 227, 327, 94, 107, 101, 44, 166, 260, 368, 350, 371, 381, 98, 200, 66, 115, 154, 28, 187, 75, 257, 384, 318, 23, 311, 200, 329, 82, 264, 151, 188, 181, 118, 103, 24, 147, 301, 335, 162, 274, 6, 227, 364, 326, 399, 53, 168, 373, 264, 247, 192, 284, 0, 67, 237, 8, 273, 269, 118, 279, 124, 283, 98, 127, 294, 174, 373, 136, 174, 374, 233, 270, 222, 181, 326, 233, 157, 84, 103, 312, 25, 42, 235, 330, 257, 327, 155, 142, 168, 154, 395, 368, 294, 262, 286, 258, 160, 266, 102, 314, 262, 268, 99, 275, 47, 73, 129, 189, 281, 122, 162, 202, 363, 132, 129, 330, 224, 232, 11, 261, 18, 317, 189, 64, 215, 189, 105, 262, 256, 50, 130, 99, 176, 104, 338, 395, 392, 154, 170, 396, 96, 278, 162, 386, 139, 98, 326, 210, 342, 382, 116, 40, 305, 81, 321, 252, 6, 21, 201, 383, 94, 382, 184, 97, 86, 395, 216, 94, 347, 348, 281, 145, 84, 262, 23, 399, 178, 330, 398, 167, 243, 265, 241, 152, 307, 90, 272, 371, 152, 258, 228, 323, 231, 40, 82, 316, 10, 18, 102, 115, 386, 149, 217, 255, 238, 183, 391, 186, 56, 21, 339, 173, 5, 164, 224, 164, 40, 215, 380, 63, 13, 285, 69, 162, 360])
scrabler.append([400, 351, 129, 157, 133, 56, 265, 16, 130, 193, 356, 377, 341, 370, 151, 120, 275, 204, 93, 398, 124, 317, 215, 189, 155, 353, 201, 127, 230, 341, 84, 118, 209, 195, 279, 27, 327, 348, 344, 42, 105, 196, 400, 140, 245, 24, 18, 124, 173, 190, 182, 366, 182, 387, 349, 272, 329, 219, 279, 287, 315, 24, 227, 127, 299, 103, 274, 187, 162, 54, 102, 384, 391, 396, 144, 283, 328, 198, 32, 4, 246, 175, 156, 390, 316, 388, 354, 243, 103, 98, 4, 167, 273, 396, 138, 322, 263, 28, 315, 237, 206, 138, 318, 148, 148, 353, 335, 155, 231, 228, 259, 382, 89, 7, 121, 74, 334, 127, 157, 91, 329, 48, 197, 392, 327, 331, 339, 253, 138, 266, 21, 24, 141, 35, 328, 297, 303, 348, 189, 296, 161, 30, 272, 78, 25, 201, 23, 353, 236, 231, 138, 153, 384, 400, 241, 295, 125, 146, 296, 143, 313, 243, 346, 136, 68, 3, 25, 159, 20, 59, 0, 60, 261, 396, 289, 208, 123, 6, 125, 194, 90, 39, 261, 71, 345, 335, 319, 85, 206, 197, 361, 117, 152, 165, 27, 298, 146, 208, 188, 51, 350, 329, 227, 26, 142, 400, 209, 358, 125, 237, 231, 159, 193, 183, 202, 350, 124, 194, 171, 162, 8, 2, 100, 225, 51, 37, 38, 361, 330, 212, 9, 255, 59, 331, 360, 106, 189, 139, 281, 387, 261, 146, 355, 222, 1, 359, 162, 226, 163, 276, 202, 53, 203, 99, 57, 174, 71, 159, 60, 288, 47, 59, 195, 381, 277, 9, 191, 47, 107, 290, 155, 381, 21, 39, 140, 280, 397, 383, 180, 199, 280, 238, 397, 178, 73, 254, 296, 111, 337, 299, 217, 107, 362, 345, 161, 141, 393, 150, 156, 100, 12, 317, 319, 205, 386, 203, 391, 319, 32, 219, 116, 121, 72, 239, 136, 135, 204, 119, 114, 53, 225, 249, 249, 294, 120, 383, 97, 136, 220, 276, 172, 261, 327, 34, 6, 181, 25, 198, 105, 101, 347, 179, 32, 366, 179, 62, 201, 329, 391, 68, 133, 25, 23, 250, 353, 127, 275, 94, 47, 271, 341, 37, 351, 314, 7, 51, 260, 2, 53, 231, 353, 331, 86, 78, 7, 107, 187, 38, 310, 203, 41, 97, 103, 391, 336, 324, 294, 258, 247, 253, 95, 81, 205, 252, 334, 53, 358, 156, 352, 346])
scrabler.append([386, 265, 143, 379, 69, 279, 53, 373, 196, 387, 368, 205, 336, 361, 185, 344, 68, 320, 368, 58, 213, 24, 190, 276, 392, 47, 31, 162, 238, 288, 250, 106, 117, 291, 389, 302, 58, 375, 396, 120, 10, 241, 290, 213, 332, 385, 142, 364, 399, 151, 174, 249, 251, 266, 7, 348, 92, 333, 63, 129, 338, 103, 188, 41, 280, 331, 299, 18, 336, 250, 179, 73, 264, 40, 372, 382, 68, 45, 346, 13, 197, 178, 27, 257, 383, 202, 248, 313, 248, 179, 293, 239, 118, 384, 235, 327, 375, 274, 272, 359, 224, 329, 248, 260, 165, 368, 356, 62, 73, 72, 9, 74, 94, 182, 164, 23, 345, 321, 330, 389, 258, 256, 316, 311, 8, 7, 112, 245, 326, 32, 44, 393, 383, 262, 26, 188, 14, 188, 301, 143, 23, 294, 44, 124, 141, 294, 213, 127, 248, 350, 30, 329, 384, 248, 184, 207, 388, 153, 216, 196, 328, 293, 188, 86, 380, 391, 264, 152, 89, 184, 378, 119, 47, 162, 36, 169, 357, 135, 156, 285, 137, 250, 212, 30, 213, 21, 224, 355, 389, 360, 160, 160, 345, 331, 145, 104, 172, 332, 241, 78, 195, 157, 350, 226, 183, 334, 257, 27, 167, 313, 350, 340, 237, 386, 366, 392, 87, 66, 50, 39, 90, 166, 180, 352, 173, 248, 185, 358, 42, 300, 159, 320, 17, 199, 82, 22, 7, 32, 293, 109, 291, 73, 382, 255, 275, 323, 84, 264, 216, 33, 14, 138, 21, 133, 278, 299, 179, 316, 65, 381, 231, 391, 220, 314, 298, 395, 139, 8, 212, 225, 157, 34, 11, 394, 162, 270, 371, 111, 32, 117, 190, 193, 73, 310, 389, 295, 303, 305, 284, 166, 281, 100, 177, 400, 376, 391, 284, 207, 50, 345, 120, 276, 4, 323, 289, 121, 249, 371, 14, 86, 221, 382, 271, 10, 69, 126, 179, 155, 346, 161, 72, 185, 370, 78, 306, 384, 209, 232, 109, 31, 14, 95, 26, 62, 382, 332, 341, 68, 7, 385, 116, 195, 339, 165, 365, 314, 100, 229, 95, 43, 229, 352, 71, 144, 332, 164, 180, 41, 174, 202, 342, 313, 30, 12, 328, 9, 120, 201, 362, 97, 323, 394, 64, 225, 220, 47, 75, 117, 189, 348, 229, 171, 227, 314, 366, 300, 189, 89, 377, 340, 108, 400, 192, 118, 143, 39, 390, 390, 196, 210])
scrabler.append([105, 303, 326, 339, 306, 90, 380, 378, 25, 28, 117, 308, 331, 21, 176, 383, 70, 179, 182, 327, 376, 142, 284, 65, 196, 397, 71, 12, 282, 3, 112, 77, 78, 75, 240, 180, 117, 48, 372, 104, 101, 170, 241, 332, 165, 60, 374, 33, 186, 246, 315, 19, 62, 321, 47, 353, 20, 123, 82, 28, 248, 213, 135, 337, 58, 132, 39, 3, 341, 270, 229, 337, 65, 303, 146, 317, 165, 382, 57, 148, 321, 216, 69, 296, 317, 27, 137, 101, 168, 275, 256, 31, 97, 202, 139, 331, 236, 52, 353, 76, 398, 68, 31, 222, 332, 270, 314, 362, 48, 332, 190, 97, 125, 191, 154, 326, 91, 109, 292, 90, 301, 197, 242, 157, 385, 134, 69, 114, 19, 168, 123, 212, 136, 238, 188, 312, 397, 121, 102, 45, 375, 39, 100, 116, 120, 16, 317, 227, 185, 112, 29, 1, 70, 309, 150, 40, 25, 253, 140, 141, 276, 266, 265, 72, 138, 247, 296, 208, 3, 283, 116, 225, 385, 106, 122, 373, 277, 89, 86, 169, 18, 186, 206, 49, 146, 10, 258, 328, 271, 97, 393, 113, 234, 113, 36, 319, 356, 394, 399, 286, 382, 314, 284, 330, 223, 99, 225, 367, 131, 143, 389, 51, 62, 249, 355, 101, 342, 183, 185, 135, 104, 137, 60, 290, 91, 118, 287, 399, 322, 117, 78, 96, 199, 45, 235, 350, 180, 10, 317, 54, 279, 184, 171, 178, 354, 375, 85, 260, 384, 270, 17, 114, 114, 205, 47, 104, 302, 248, 181, 393, 11, 294, 1, 71, 312, 205, 135, 272, 20, 39, 362, 37, 33, 363, 242, 390, 10, 254, 55, 90, 273, 59, 338, 387, 241, 74, 170, 71, 200, 277, 117, 159, 316, 255, 236, 83, 161, 394, 306, 105, 392, 70, 10, 164, 81, 227, 191, 68, 62, 373, 5, 59, 5, 275, 302, 202, 45, 354, 171, 5, 251, 400, 97, 44, 303, 202, 235, 375, 311, 284, 61, 65, 287, 111, 53, 150, 331, 162, 118, 373, 115, 185, 197, 220, 314, 76, 107, 348, 25, 103, 158, 81, 141, 325, 87, 359, 82, 142, 236, 56, 179, 192, 2, 46, 10, 392, 135, 253, 317, 334, 117, 322, 135, 50, 123, 347, 87, 398, 5, 282, 266, 285, 320, 156, 68, 82, 188, 256, 377, 136, 183, 340, 274, 29, 299, 394, 332, 350, 141, 37])
scrabler.append([292, 201, 220, 27, 375, 166, 164, 169, 284, 192, 176, 264, 26, 338, 78, 10, 321, 81, 70, 144, 110, 100, 257, 397, 155, 2, 400, 341, 72, 15, 204, 83, 111, 146, 52, 40, 362, 205, 387, 28, 251, 73, 5, 124, 323, 186, 398, 206, 163, 47, 335, 386, 24, 98, 298, 75, 379, 134, 213, 391, 230, 322, 55, 29, 82, 17, 270, 299, 222, 364, 12, 316, 125, 99, 60, 60, 101, 19, 271, 5, 184, 200, 139, 291, 274, 87, 22, 352, 158, 200, 318, 281, 70, 197, 29, 270, 109, 257, 330, 214, 261, 140, 375, 63, 63, 359, 166, 265, 324, 104, 152, 327, 196, 320, 297, 109, 358, 158, 372, 110, 342, 169, 175, 71, 93, 335, 50, 139, 209, 64, 391, 349, 203, 187, 278, 326, 214, 214, 182, 228, 269, 58, 299, 324, 183, 159, 333, 265, 158, 155, 53, 30, 113, 75, 397, 33, 205, 299, 173, 241, 141, 298, 35, 24, 327, 255, 286, 205, 176, 275, 70, 357, 37, 188, 163, 33, 46, 132, 62, 211, 262, 42, 186, 183, 93, 120, 210, 54, 194, 240, 372, 351, 98, 388, 287, 20, 258, 246, 298, 208, 339, 217, 17, 372, 283, 267, 227, 356, 389, 33, 209, 330, 12, 363, 311, 215, 366, 178, 275, 92, 363, 229, 114, 318, 67, 343, 88, 347, 238, 30, 36, 124, 28, 200, 52, 394, 72, 269, 155, 148, 22, 67, 28, 301, 264, 339, 393, 397, 27, 71, 395, 219, 176, 70, 332, 384, 78, 147, 333, 322, 26, 143, 73, 276, 265, 309, 79, 42, 50, 346, 121, 179, 373, 153, 241, 369, 398, 274, 173, 178, 329, 240, 377, 28, 132, 316, 167, 178, 197, 53, 120, 76, 31, 116, 369, 303, 17, 13, 398, 295, 248, 335, 102, 397, 127, 373, 203, 15, 171, 371, 233, 225, 344, 113, 331, 86, 191, 347, 369, 223, 144, 385, 47, 367, 66, 210, 344, 98, 0, 330, 160, 129, 205, 283, 128, 33, 217, 124, 338, 18, 148, 282, 382, 122, 112, 80, 199, 225, 57, 341, 139, 63, 72, 322, 205, 288, 379, 86, 79, 375, 357, 75, 368, 125, 100, 290, 309, 397, 36, 209, 142, 61, 241, 153, 123, 199, 174, 179, 287, 120, 212, 102, 350, 84, 2, 367, 10, 221, 291, 38, 72, 164, 362, 80, 69, 393, 359, 330, 268, 39])
scrabler.append([189, 226, 370, 277, 3, 111, 184, 343, 210, 273, 20, 222, 396, 133, 149, 182, 228, 265, 102, 234, 323, 50, 191, 201, 337, 58, 348, 345, 101, 234, 399, 376, 315, 355, 212, 115, 145, 160, 100, 264, 14, 292, 382, 356, 322, 346, 388, 58, 287, 144, 265, 341, 246, 87, 60, 316, 205, 306, 192, 306, 316, 75, 182, 303, 48, 169, 358, 253, 395, 385, 193, 271, 329, 395, 108, 56, 172, 364, 169, 333, 301, 172, 31, 255, 358, 100, 114, 345, 350, 51, 206, 214, 93, 236, 364, 181, 259, 366, 74, 390, 109, 302, 246, 352, 7, 344, 87, 286, 382, 355, 167, 190, 119, 398, 314, 42, 345, 186, 379, 51, 108, 29, 191, 183, 160, 237, 242, 88, 175, 179, 388, 53, 395, 365, 273, 400, 176, 9, 379, 161, 30, 304, 20, 381, 351, 141, 203, 395, 223, 157, 4, 310, 189, 378, 36, 380, 334, 323, 258, 263, 10, 150, 34, 354, 343, 337, 125, 45, 236, 394, 106, 207, 285, 377, 371, 381, 282, 381, 385, 359, 32, 185, 242, 287, 269, 23, 394, 180, 195, 228, 156, 153, 173, 363, 292, 279, 318, 362, 126, 11, 376, 205, 132, 40, 351, 329, 253, 119, 161, 194, 83, 395, 48, 335, 311, 190, 170, 17, 145, 55, 303, 145, 303, 35, 256, 124, 29, 251, 293, 210, 326, 351, 323, 210, 312, 223, 302, 277, 0, 285, 200, 285, 136, 354, 78, 255, 44, 16, 185, 11, 74, 159, 10, 241, 339, 191, 14, 227, 32, 366, 251, 107, 192, 383, 96, 257, 293, 91, 391, 141, 334, 262, 280, 224, 228, 170, 348, 34, 31, 122, 203, 360, 260, 72, 55, 107, 171, 258, 29, 370, 335, 92, 243, 342, 326, 156, 124, 300, 14, 235, 303, 179, 304, 284, 9, 152, 259, 166, 182, 76, 175, 398, 78, 21, 179, 286, 0, 365, 184, 237, 14, 246, 381, 203, 35, 49, 70, 235, 400, 173, 265, 92, 191, 8, 317, 188, 293, 232, 28, 329, 148, 297, 119, 206, 212, 27, 5, 134, 350, 330, 151, 277, 208, 72, 60, 393, 24, 52, 87, 257, 96, 55, 31, 274, 217, 30, 345, 174, 386, 197, 239, 112, 86, 353, 300, 284, 339, 123, 398, 68, 384, 225, 307, 58, 213, 74, 313, 62, 70, 166, 32, 170, 270, 333, 213, 124, 385, 198, 255, 90])

def get(iscramble, index, mix):
    # Get scrambler
    scramble = scrabler[iscramble]
    
    # Index length of scrambler
    index = index % len(scramble)
    index = scramble[index]
    index = index % len(mix.handles)
    
    # Return handle
    return mix.handles[index]

def times_name(handle, mutation, mix=None, size=None):
    # Prefix is mutation
    name = mutation
    
    # Add size if given
    if size is not None:
        name += '_' + size.name
    
    # Add mix if given
    if mix is not None:
        name += '_' + mix.name
        
    # Handle is required
    name += '_' + handle.name
      
    return name 

if __name__ == '__main__':
    # Create a new scrambler
    for _ in xrange(50):
        rset = []
        for _ in xrange(__WORKLOAD_MAX_RANDOM):
            value = random.randint(0, __WORKLOAD_MAX_RANDOM)
            rset.append(value)
        print 'scrabler.append(%s)' % rset
